"""GitHub models provider."""

from __future__ import annotations

import logging
import os
from typing import TYPE_CHECKING, Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


logger = logging.getLogger(__name__)


def get_token_from_gh_cli() -> str | None:
    """Get GitHub token from gh CLI."""
    import subprocess

    try:
        result = subprocess.run(
            ["gh", "auth", "token"], capture_output=True, text=True, check=True
        )
        token = result.stdout.strip()
    except (subprocess.SubprocessError, FileNotFoundError) as e:
        logger.debug("Failed to get GitHub token from gh CLI: %s", e)
        return None
    else:
        return token if token else None


class GitHubProvider(ModelProvider):
    """GitHub AI models API provider."""

    def __init__(self, token: str | None = None):
        super().__init__()
        self.token = token or os.environ.get("GITHUB_TOKEN")
        if not self.token:
            self.token = get_token_from_gh_cli()

        if not self.token:
            msg = "GitHub token not found in parameters, GITHUB_TOKEN env var, or gh CLI"
            raise RuntimeError(msg)

        self.base_url = "https://api.catalog.azureml.ms"
        self.models_url = f"{self.base_url}/asset-gallery/v1.0/models"
        self.headers = {
            "Content-Type": "application/json",
            "Authorization": f"Bearer {self.token}",
        }

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.token)

    async def _fetch_models(self, is_free: bool) -> list[ModelInfo]:
        """Fetch models with specific freePlayground value."""
        from anyenv import post

        params = {
            "filters": [
                {
                    "field": "freePlayground",
                    "values": ["true" if is_free else "false"],
                    "operator": "eq",
                },
                {"field": "labels", "values": ["latest"], "operator": "eq"},
            ],
            "order": [{"field": "displayName", "direction": "asc"}],
        }

        try:
            response = await post(
                self.models_url,
                json=params,
                headers=self.headers,
                cache=True,
            )
            data = await response.json()
            if not isinstance(data, dict) or "summaries" not in data:
                msg = "Invalid response format from GitHub Models API"
                raise RuntimeError(msg)  # noqa: TRY301

            # Inject is_free into each model's data
            summaries = []
            for item in data["summaries"]:
                item_with_free = item.copy()  # Make a copy to not modify the original
                item_with_free["is_free"] = is_free
                summaries.append(item_with_free)

            return [self._parse_model(item) for item in summaries]
        except Exception as e:
            msg = f"Failed to fetch models from GitHub: {e}"
            logger.exception(msg)
            raise RuntimeError(msg) from e

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse GitHub models API response into ModelInfo."""
        inference_task = ""
        if data.get("task"):
            inference_task = data["task"]
        elif data.get("inferenceTasks") and isinstance(data["inferenceTasks"], list):
            inference_task = data["inferenceTasks"][0] if data["inferenceTasks"] else ""

        # Combine summary and task information for description
        description = data.get("summary", "")
        if inference_task:
            description = (
                f"{description}\nTask: {inference_task}"
                if description
                else f"Task: {inference_task}"
            )

        # Extract context window and output tokens
        context_window = None
        max_output_tokens = None
        limits = data.get("modelLimits")
        if isinstance(limits, dict) and isinstance(limits.get("textLimits"), dict):
            text_limits = limits["textLimits"]
            context_window = text_limits.get("inputContextWindow")
            max_output_tokens = text_limits.get("maxOutputTokens")

        input_modalities: set[Modality] = {"text"}
        output_modalities: set[Modality] = {"text"}
        if isinstance(limits, dict):
            if limits.get("supportedInputModalities"):
                input_modalities = set(limits["supportedInputModalities"])
            if limits.get("supportedOutputModalities"):
                output_modalities = set(limits["supportedOutputModalities"])

        # Use name as ID as it's more consistent with other providers
        model_id = data.get("name", "")
        model_name = (
            data.get("friendly_name", "") or data.get("displayName", "") or model_id
        )

        return ModelInfo(
            id=model_id,
            name=model_name,
            provider="github",
            description=description,
            owned_by=data.get("publisher", "GitHub"),
            context_window=context_window,
            max_output_tokens=max_output_tokens,
            input_modalities=input_modalities,
            output_modalities=output_modalities,
            is_free=data.get("is_free", True),
        )

    async def get_models(self) -> list[ModelInfo]:
        """Get both free and paid models through separate API calls."""
        free_models = await self._fetch_models(is_free=True)
        paid_models = await self._fetch_models(is_free=False)
        return free_models + paid_models


if __name__ == "__main__":
    import asyncio

    async def main():
        provider = GitHubProvider()
        models = await provider.get_models()
        for model in models:
            print(model.format())

    asyncio.run(main())



"""Model discovery and information retrieval."""

from __future__ import annotations

from dataclasses import dataclass, field
from typing import Any, Literal


Modality = Literal["text", "image", "audio", "video"]


@dataclass
class ModelPricing:
    """Pricing information for a model."""

    prompt: float | None = None
    """Cost per token for prompt inputs."""
    completion: float | None = None
    """Cost per token for completion outputs."""
    image: float | None = None
    """Cost per image for input processing."""
    request: float | None = None
    """Cost per request."""
    input_cache_read: float | None = None
    """Cost for reading from input cache."""
    input_cache_write: float | None = None
    """Cost for writing to input cache."""
    web_search: float | None = None
    """Cost for web search functionality."""
    internal_reasoning: float | None = None
    """Cost for internal reasoning functionality."""


@dataclass
class ModelInfo:
    """Unified model information from various providers."""

    id: str
    """Unique identifier for the model."""
    name: str
    """Display name of the model."""
    provider: str
    """Service provider name (e.g. OpenAI, Anthropic)."""
    description: str | None = None
    """Detailed description of the model's capabilities."""
    pricing: ModelPricing | None = None
    """Pricing information for using the model."""
    owned_by: str | None = None
    """Organization that owns/created the model."""
    context_window: int | None = None
    """Maximum number of tokens that can be processed in one request."""
    is_deprecated: bool = False
    """Whether this model version is deprecated."""
    is_embedding: bool = False
    """Whether this model is primarily used for creating embeddings."""
    max_output_tokens: int | None = None
    """Maximum number of tokens the model can generate in a response."""
    input_modalities: set[Modality] = field(default_factory=lambda: {"text"})
    """Supported input modalities (text, image, audio, video, etc.)."""
    output_modalities: set[Modality] = field(default_factory=lambda: {"text"})
    """Supported output modalities (text, image, audio, video, etc.)."""
    is_free: bool = False
    """Whether this model is free to use."""
    metadata: dict[str, Any] = field(default_factory=dict)
    """Provider-specific metadata that doesn't fit in standard fields."""

    @property
    def pydantic_ai_id(self) -> str:
        """Unique pydantic-ai style identifier for the model."""
        return f"{self.provider}:{self.id}"

    @property
    def litellm_id(self) -> str:
        """Unique litellm style identifier for the model."""
        return f"{self.provider}/{self.id}"

    @property
    def iconify_icon(self) -> str | None:  # noqa: PLR0911
        """Iconify icon for the model."""
        name = self.name.lower()
        if name.startswith("mistral"):
            return "logos:mistral-ai"
        if name.startswith("openai"):
            return "logos:openai"
        if name.startswith("claude"):
            return "logos:anthropic"
        if name.startswith("perplexity"):
            return "logos:perplexity"
        if name.startswith("hugging"):
            return "logos:hugging-face"
        if name.startswith("deepseek"):
            return "arcticons:deepseek"

        return None

    def format(self) -> str:
        """Format model information as a human-readable string.

        Returns:
            str: Formatted model information
        """
        lines: list[str] = []

        # Basic info
        lines.append(f"Model: {self.name}")
        lines.append(f"Provider: {self.provider}")
        lines.append(f"ID: {self.id}")

        # Optional fields
        if self.owned_by:
            lines.append(f"Owned by: {self.owned_by}")

        if self.context_window:
            lines.append(f"Context window: {self.context_window:,} tokens")

        if self.max_output_tokens:
            lines.append(f"Max output tokens: {self.max_output_tokens:,}")

        if self.pricing:
            if self.pricing.prompt is not None:
                lines.append(f"Prompt cost: ${self.pricing.prompt:.6f}/token")
            if self.pricing.completion is not None:
                lines.append(f"Completion cost: ${self.pricing.completion:.6f}/token")
            if self.pricing.image is not None and self.pricing.image > 0:
                lines.append(f"Image cost: ${self.pricing.image:.6f}/image")
            if self.pricing.request is not None and self.pricing.request > 0:
                lines.append(f"Request cost: ${self.pricing.request:.6f}/request")
            if self.pricing.web_search is not None and self.pricing.web_search > 0:
                lines.append(f"Web search cost: ${self.pricing.web_search:.6f}")
            if (
                self.pricing.internal_reasoning is not None
                and self.pricing.internal_reasoning > 0
            ):
                lines.append(
                    f"Internal reasoning cost: ${self.pricing.internal_reasoning:.6f}"
                )

        if self.input_modalities and self.input_modalities != {"text"}:
            lines.append(f"Input modalities: {', '.join(self.input_modalities)}")

        if self.output_modalities and self.output_modalities != {"text"}:
            lines.append(f"Output modalities: {', '.join(self.output_modalities)}")

        if self.description:
            lines.append("\nDescription:")
            lines.append(self.description)

        if self.is_deprecated:
            lines.append("\n⚠️ This model is deprecated")

        # Add any relevant metadata
        if self.metadata and any(self.metadata.values()):
            lines.append("\nAdditional Information:")
            for key, value in self.metadata.items():
                if key == "supported_parameters" and value:
                    lines.append(
                        f"- Supported parameters: {', '.join(str(v) for v in value)}"
                    )
                elif key == "hugging_face_id" and value:
                    lines.append(f"- HuggingFace ID: {value}")
                elif key == "tokenizer" and value:
                    lines.append(f"- Tokenizer: {value}")
                elif key == "is_moderated" and value:
                    lines.append(f"- Moderated: {value}")
                elif key == "created_timestamp" and value:
                    from datetime import datetime

                    timestamp = datetime.fromtimestamp(value).strftime("%Y-%m-%d")
                    lines.append(f"- Created: {timestamp}")
                elif isinstance(value, list | set) and value:
                    vals = ", ".join(str(v) for v in value)
                    lines.append(f"- {key.replace('_', ' ').title()}: {vals}")
                elif value:
                    lines.append(f"- {key.replace('_', ' ').title()}: {value}")

        return "\n".join(lines)



"""Ollama provider."""

from __future__ import annotations

import socket
from typing import TYPE_CHECKING, Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo, ModelPricing


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


class OllamaProvider(ModelProvider):
    """Ollama local API provider."""

    def __init__(self, base_url: str = "http://localhost:11434"):
        super().__init__()
        self.base_url = base_url
        self.headers = {}
        self.params = {}

    def is_available(self) -> bool:
        """Check whether Ollama is running and available."""
        try:
            # Extract host and port from base_url
            if "://" in self.base_url:
                host_port = self.base_url.split("://")[1]
            else:
                host_port = self.base_url

            if ":" in host_port:
                host, port_str = host_port.split(":", 1)
                port = int(port_str.split("/")[0])  # Handle paths after port
            else:
                host = host_port
                port = 11434  # Default Ollama port

            # Try to connect to the Ollama server
            with socket.create_connection((host, port), timeout=2):
                return True
        except (ValueError, OSError):
            return False

    async def get_models(self) -> list[ModelInfo]:
        """Fetch available models from Ollama using the /api/tags endpoint."""
        from anyenv import HttpError, get_json, post_json

        try:
            data = await get_json(
                f"{self.base_url}/api/tags",
                headers=self.headers,
                params=self.params,
                cache=True,
                return_type=dict,
            )

            if not isinstance(data, dict) or "models" not in data:
                msg = f"Invalid response format from {self.__class__.__name__}"
                raise RuntimeError(msg)

            # Fetch detailed info for each model
            models = []
            for item in data["models"]:
                try:
                    # Get detailed model info using /api/show
                    detail_data = await post_json(
                        f"{self.base_url}/api/show",
                        json_data={"name": item["name"]},
                        headers=self.headers,
                        cache=True,
                        return_type=dict,
                    )
                    models.append(self._parse_model(item, detail_data))
                except HttpError:
                    # Fallback to basic info if detailed fetch fails
                    models.append(self._parse_model(item))
        except HttpError as e:
            msg = f"Failed to fetch models from {self.__class__.__name__}: {e}"
            raise RuntimeError(msg) from e
        else:
            return models

    def _parse_model(
        self, data: dict[str, Any], detail_data: dict[str, Any] | None = None
    ) -> ModelInfo:
        """Parse Ollama API response into ModelInfo."""
        name = str(data["name"])

        # Extract model details from basic info
        details = data.get("details", {})
        family = details.get("family", "")
        parameter_size = details.get("parameter_size", "")
        quantization = details.get("quantization_level", "")

        # Extract additional info from detailed response if available
        context_window = None
        exact_param_count = None
        model_parameters = {}

        if detail_data:
            model_info = detail_data.get("model_info", {})

            # Get actual context window from model_info
            for key, value in model_info.items():
                if key.endswith(".context_length"):
                    context_window = int(value)
                    break

            # Get exact parameter count
            exact_param_count = model_info.get("general.parameter_count")

            # Parse model parameters (temperature, top_k, etc.)
            params_str = detail_data.get("parameters", "")
            if params_str:
                for line in params_str.strip().split("\n"):
                    if line.strip():
                        parts = line.strip().split(None, 1)
                        if len(parts) == 2:  # noqa: PLR2004
                            param_name, param_value = parts
                            # Convert numeric values
                            try:
                                if "." in param_value.strip('"'):
                                    model_parameters[param_name] = float(
                                        param_value.strip('"')
                                    )
                                elif param_value.strip('"').isdigit():
                                    model_parameters[param_name] = int(
                                        param_value.strip('"')
                                    )
                                else:
                                    model_parameters[param_name] = param_value.strip('"')
                            except ValueError:
                                model_parameters[param_name] = param_value.strip('"')

        # Fallback to estimation if no detailed context window found
        if not context_window:
            context_window = self._estimate_context_window(
                parameter_size, data.get("size", 0)
            )

        # Build description from available details
        description_parts = []
        if family:
            description_parts.append(f"Family: {family}")
        if parameter_size:
            description_parts.append(f"Parameters: {parameter_size}")
        if exact_param_count and exact_param_count != parameter_size:
            description_parts.append(f"Exact count: {exact_param_count:,}")
        if quantization:
            description_parts.append(f"Quantization: {quantization}")

        description = " | ".join(description_parts) if description_parts else None

        # Determine modalities based on family
        input_modalities: set[Modality] = {"text"}
        output_modalities: set[Modality] = {"text"}

        if (family and "llava" in family.lower()) or (
            family
            and any(
                vision in family.lower() for vision in ["vision", "visual", "multimodal"]
            )
        ):
            input_modalities.add("image")

        # Build comprehensive metadata
        metadata = {
            "size_bytes": data.get("size", 0),
            "digest": data.get("digest"),
            "modified_at": data.get("modified_at"),
            "format": details.get("format"),
            "families": details.get("families", []),
            "quantization_level": quantization,
            "parameter_size": parameter_size,
            "parent_model": details.get("parent_model"),
        }

        # Add detailed info if available
        if detail_data:
            metadata.update({
                "exact_parameter_count": exact_param_count,
                "model_parameters": model_parameters,
                "has_template": bool(detail_data.get("template")),
                "license_type": "Apache License 2.0"
                if "Apache License" in detail_data.get("license", "")
                else None,
            })

            # Add model architecture details from model_info
            if model_info := detail_data.get("model_info", {}):
                arch_info = {}
                for key, value in model_info.items():
                    if any(
                        key.startswith(prefix) for prefix in ["general.", f"{family}."]
                    ):
                        clean_key = key.split(".", 1)[1] if "." in key else key
                        arch_info[clean_key] = value
                if arch_info:
                    metadata["architecture"] = arch_info

        return ModelInfo(
            id=name,
            name=name,
            provider="ollama",
            description=description,
            pricing=ModelPricing(),  # Local models are free
            owned_by=family if family else None,
            context_window=context_window,
            is_free=True,  # Local Ollama models are free to use
            input_modalities=input_modalities,
            output_modalities=output_modalities,
            metadata=metadata,
        )

    def _estimate_context_window(
        self, parameter_size: str, size_bytes: int
    ) -> int | None:
        """Estimate context window based on parameter size and model family."""
        if not parameter_size:
            return None

        param_size_lower = parameter_size.lower()

        # Common context windows for different model sizes
        if (
            "1b" in param_size_lower
            or "1.1b" in param_size_lower
            or "3b" in param_size_lower
            or "7b" in param_size_lower
            or "8b" in param_size_lower
        ):
            return 8192
        if (
            "13b" in param_size_lower
            or "14b" in param_size_lower
            or "30b" in param_size_lower
            or "34b" in param_size_lower
            or "70b" in param_size_lower
        ):
            return 4096
        if "405b" in param_size_lower:
            return 8192

        # Default fallback
        return 4096


if __name__ == "__main__":
    import asyncio

    provider = OllamaProvider()
    if provider.is_available():
        models = asyncio.run(provider.get_models())
        for model in models:
            print(model.format())
            print("-" * 50)
    else:
        print("Ollama is not running or not available")



"""Generated model names for requesty provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 320
"""

from __future__ import annotations

from typing import Literal


RequestyModels = Literal[
    "requesty:alibaba/qwen-max",
    "requesty:alibaba/qwen-plus",
    "requesty:alibaba/qwen-turbo",
    "requesty:alibaba/qwen3-30b-a3b-instruct-2507",
    "requesty:alibaba/qwen3-coder-flash",
    "requesty:alibaba/qwen3-coder-plus",
    "requesty:alibaba/qwen3-max",
    "requesty:anthropic/claude-3-5-haiku",
    "requesty:anthropic/claude-3-7-sonnet",
    "requesty:anthropic/claude-3-haiku",
    "requesty:anthropic/claude-3-opus",
    "requesty:anthropic/claude-haiku-4-5",
    "requesty:anthropic/claude-opus-4",
    "requesty:anthropic/claude-opus-4-1",
    "requesty:anthropic/claude-sonnet-4",
    "requesty:anthropic/claude-sonnet-4-5",
    "requesty:azure/gpt-4.1",
    "requesty:azure/gpt-4.1-mini",
    "requesty:azure/gpt-4.1-mini@eastus2",
    "requesty:azure/gpt-4.1-mini@francecentral",
    "requesty:azure/gpt-4.1-mini@swedencentral",
    "requesty:azure/gpt-4.1-mini@uksouth",
    "requesty:azure/gpt-4.1-mini@westus3",
    "requesty:azure/gpt-4.1-nano",
    "requesty:azure/gpt-4.1-nano@eastus2",
    "requesty:azure/gpt-4.1-nano@francecentral",
    "requesty:azure/gpt-4.1-nano@swedencentral",
    "requesty:azure/gpt-4.1-nano@uksouth",
    "requesty:azure/gpt-4.1-nano@westus3",
    "requesty:azure/gpt-4.1@eastus2",
    "requesty:azure/gpt-4.1@francecentral",
    "requesty:azure/gpt-4.1@swedencentral",
    "requesty:azure/gpt-4.1@uksouth",
    "requesty:azure/gpt-4.1@westus3",
    "requesty:azure/gpt-5",
    "requesty:azure/gpt-5-mini",
    "requesty:azure/gpt-5-mini@eastus2",
    "requesty:azure/gpt-5-mini@swedencentral",
    "requesty:azure/gpt-5-mini@uksouth",
    "requesty:azure/gpt-5-nano",
    "requesty:azure/gpt-5-nano@eastus2",
    "requesty:azure/gpt-5-nano@swedencentral",
    "requesty:azure/gpt-5@eastus2",
    "requesty:azure/gpt-5@swedencentral",
    "requesty:azure/gpt-5@uksouth",
    "requesty:azure/o4-mini",
    "requesty:azure/o4-mini@eastus2",
    "requesty:azure/o4-mini@francecentral",
    "requesty:azure/o4-mini@swedencentral",
    "requesty:azure/o4-mini@uksouth",
    "requesty:azure/o4-mini@westus3",
    "requesty:bedrock/claude-3-7-sonnet",
    "requesty:bedrock/claude-3-7-sonnet@eu-central-1",
    "requesty:bedrock/claude-3-7-sonnet@eu-north-1",
    "requesty:bedrock/claude-3-7-sonnet@eu-west-1",
    "requesty:bedrock/claude-3-7-sonnet@eu-west-3",
    "requesty:bedrock/claude-3-7-sonnet@us-east-1",
    "requesty:bedrock/claude-3-7-sonnet@us-east-2",
    "requesty:bedrock/claude-3-7-sonnet@us-west-2",
    "requesty:bedrock/claude-haiku-4-5",
    "requesty:bedrock/claude-haiku-4-5@eu-central-1",
    "requesty:bedrock/claude-haiku-4-5@eu-north-1",
    "requesty:bedrock/claude-haiku-4-5@eu-west-1",
    "requesty:bedrock/claude-haiku-4-5@eu-west-3",
    "requesty:bedrock/claude-haiku-4-5@us-east-1",
    "requesty:bedrock/claude-haiku-4-5@us-east-2",
    "requesty:bedrock/claude-haiku-4-5@us-west-2",
    "requesty:bedrock/claude-opus-4",
    "requesty:bedrock/claude-opus-4@us-east-1",
    "requesty:bedrock/claude-opus-4@us-east-2",
    "requesty:bedrock/claude-opus-4@us-west-2",
    "requesty:bedrock/claude-sonnet-4",
    "requesty:bedrock/claude-sonnet-4-5",
    "requesty:bedrock/claude-sonnet-4-5@eu-central-1",
    "requesty:bedrock/claude-sonnet-4-5@eu-north-1",
    "requesty:bedrock/claude-sonnet-4-5@eu-west-1",
    "requesty:bedrock/claude-sonnet-4-5@eu-west-3",
    "requesty:bedrock/claude-sonnet-4-5@us-east-1",
    "requesty:bedrock/claude-sonnet-4-5@us-east-2",
    "requesty:bedrock/claude-sonnet-4-5@us-west-2",
    "requesty:bedrock/claude-sonnet-4@eu-central-1",
    "requesty:bedrock/claude-sonnet-4@eu-north-1",
    "requesty:bedrock/claude-sonnet-4@eu-west-1",
    "requesty:bedrock/claude-sonnet-4@eu-west-3",
    "requesty:bedrock/claude-sonnet-4@us-east-1",
    "requesty:bedrock/claude-sonnet-4@us-east-2",
    "requesty:bedrock/claude-sonnet-4@us-west-2",
    "requesty:coding/claude-3-7-sonnet-20250219",
    "requesty:coding/claude-3-7-sonnet-20250219:1024",
    "requesty:coding/claude-3-7-sonnet-20250219:16384",
    "requesty:coding/claude-3-7-sonnet-20250219:64000",
    "requesty:coding/claude-3-7-sonnet-20250219:8192",
    "requesty:coding/claude-3-7-sonnet-20250219:high",
    "requesty:coding/claude-3-7-sonnet-20250219:low",
    "requesty:coding/claude-3-7-sonnet-20250219:max",
    "requesty:coding/claude-3-7-sonnet-20250219:medium",
    "requesty:coding/claude-opus-4-20250514",
    "requesty:coding/claude-sonnet-4-20250514",
    "requesty:coding/gemini-2.5-flash",
    "requesty:coding/gemini-2.5-flash@europe-central2",
    "requesty:coding/gemini-2.5-flash@europe-north1",
    "requesty:coding/gemini-2.5-flash@europe-west1",
    "requesty:coding/gemini-2.5-flash@europe-west4",
    "requesty:coding/gemini-2.5-flash@europe-west8",
    "requesty:coding/gemini-2.5-flash@us-central1",
    "requesty:coding/gemini-2.5-flash@us-east1",
    "requesty:coding/gemini-2.5-flash@us-east5",
    "requesty:coding/gemini-2.5-flash@us-south1",
    "requesty:coding/gemini-2.5-flash@us-west1",
    "requesty:coding/gemini-2.5-pro",
    "requesty:coding/gemini-2.5-pro@europe-central2",
    "requesty:coding/gemini-2.5-pro@europe-north1",
    "requesty:coding/gemini-2.5-pro@europe-west1",
    "requesty:coding/gemini-2.5-pro@europe-west4",
    "requesty:coding/gemini-2.5-pro@europe-west8",
    "requesty:coding/gemini-2.5-pro@us-central1",
    "requesty:coding/gemini-2.5-pro@us-east1",
    "requesty:coding/gemini-2.5-pro@us-east5",
    "requesty:coding/gemini-2.5-pro@us-south1",
    "requesty:coding/gemini-2.5-pro@us-west1",
    "requesty:deepinfra/Qwen/Qwen2.5-72B-Instruct",
    "requesty:deepinfra/Qwen/Qwen2.5-Coder-32B-Instruct",
    "requesty:deepinfra/Qwen/Qwen3-235B-A22B",
    "requesty:deepinfra/Qwen/Qwen3-32B",
    "requesty:deepinfra/Qwen/Qwen3-Coder-480B-A35B-Instruct",
    "requesty:deepinfra/deepseek-ai/DeepSeek-R1",
    "requesty:deepinfra/deepseek-ai/DeepSeek-R1-Distill-Llama-70B",
    "requesty:deepinfra/deepseek-ai/DeepSeek-V3",
    "requesty:deepinfra/deepseek-ai/DeepSeek-V3.1",
    "requesty:deepinfra/meta-llama/Llama-3.2-90B-Vision-Instruct",
    "requesty:deepinfra/meta-llama/Llama-3.3-70B-Instruct",
    "requesty:deepinfra/meta-llama/Llama-3.3-70B-Instruct-Turbo",
    "requesty:deepinfra/meta-llama/Meta-Llama-3.1-405B-Instruct",
    "requesty:deepinfra/meta-llama/Meta-Llama-3.1-70B-Instruct",
    "requesty:deepinfra/meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
    "requesty:deepinfra/microsoft/phi-4",
    "requesty:deepinfra/zai-org/GLM-4.5",
    "requesty:deepinfra/zai-org/GLM-4.5-Air",
    "requesty:deepseek/deepseek-chat",
    "requesty:deepseek/deepseek-reasoner",
    "requesty:google/gemini-2.0-flash-001",
    "requesty:google/gemini-2.5-flash",
    "requesty:google/gemini-2.5-flash-lite-preview-06-17",
    "requesty:google/gemini-2.5-pro",
    "requesty:groq/moonshotai/Kimi-K2-Instruct-0905",
    "requesty:groq/moonshotai/kimi-k2-instruct",
    "requesty:groq/openai/gpt-oss-120b",
    "requesty:groq/openai/gpt-oss-20b",
    "requesty:minimaxi/MiniMax-M2",
    "requesty:mistral/codestral-latest",
    "requesty:mistral/devstral-medium-2507",
    "requesty:mistral/devstral-small-2507",
    "requesty:mistral/devstral-small-latest",
    "requesty:mistral/mistral-large-latest",
    "requesty:mistral/mistral-medium-latest",
    "requesty:mistral/mistral-small-latest",
    "requesty:mistral/open-mistral-7b",
    "requesty:mistral/pixtral-large-latest",
    "requesty:moonshot/kimi-k2-0711-preview",
    "requesty:moonshot/kimi-k2-turbo-preview",
    "requesty:nebius/Qwen/Qwen3-Coder-480B-A35B-Instruct",
    "requesty:nebius/deepseek-ai/DeepSeek-R1-0528",
    "requesty:nebius/deepseek-ai/DeepSeek-V3-0324",
    "requesty:nebius/deepseek-ai/DeepSeek-V3-0324-fast",
    "requesty:nebius/meta-llama/Llama-3.3-70B-Instruct",
    "requesty:nebius/moonshotai/Kimi-K2-Instruct",
    "requesty:nebius/zai-org/GLM-4.5",
    "requesty:novita/Sao10K/L3-8B-Stheno-v3.2",
    "requesty:novita/deepseek/deepseek-prover-v2-671b",
    "requesty:novita/deepseek/deepseek-r1",
    "requesty:novita/deepseek/deepseek-r1-distill-llama-70b",
    "requesty:novita/deepseek/deepseek-r1-distill-qwen-14b",
    "requesty:novita/deepseek/deepseek-r1-distill-qwen-32b",
    "requesty:novita/deepseek/deepseek-r1-turbo",
    "requesty:novita/deepseek/deepseek-v3-0324",
    "requesty:novita/deepseek/deepseek-v3-turbo",
    "requesty:novita/deepseek/deepseek_v3",
    "requesty:novita/gryphe/mythomax-l2-13b",
    "requesty:novita/meta-llama/llama-3-70b-instruct",
    "requesty:novita/meta-llama/llama-3-8b-instruct",
    "requesty:novita/meta-llama/llama-3.1-8b-instruct",
    "requesty:novita/meta-llama/llama-3.2-1b-instruct",
    "requesty:novita/meta-llama/llama-3.2-3b-instruct",
    "requesty:novita/meta-llama/llama-3.3-70b-instruct",
    "requesty:novita/meta-llama/llama-4-maverick-17b-128e-instruct-fp8",
    "requesty:novita/microsoft/wizardlm-2-8x22b",
    "requesty:novita/mistralai/mistral-nemo",
    "requesty:novita/moonshotai/kimi-k2-instruct",
    "requesty:novita/nousresearch/hermes-2-pro-llama-3-8b",
    "requesty:novita/qwen/qwen-2.5-72b-instruct",
    "requesty:novita/qwen/qwen2.5-vl-72b-instruct",
    "requesty:novita/qwen/qwen3-235b-a22b-fp8",
    "requesty:novita/sao10k/l3-70b-euryale-v2.1",
    "requesty:novita/sao10k/l3-8b-lunaris",
    "requesty:novita/sao10k/l31-70b-euryale-v2.2",
    "requesty:novita/zai-org/glm-4.5",
    "requesty:novita/zai-org/glm-4.6",
    "requesty:openai-responses/gpt-5-codex",
    "requesty:openai-responses/gpt-5-mini",
    "requesty:openai-responses/gpt-5-nano",
    "requesty:openai-responses/gpt-5-pro",
    "requesty:openai-responses/o3-mini",
    "requesty:openai-responses/o3-pro",
    "requesty:openai-responses/o4-mini",
    "requesty:openai/chatgpt-4o",
    "requesty:openai/gpt-4.1",
    "requesty:openai/gpt-4.1-mini",
    "requesty:openai/gpt-4.1-nano",
    "requesty:openai/gpt-4o",
    "requesty:openai/gpt-4o-2024-05-13",
    "requesty:openai/gpt-4o-2024-08-06",
    "requesty:openai/gpt-4o-2024-11-20",
    "requesty:openai/gpt-4o-mini",
    "requesty:openai/gpt-5",
    "requesty:openai/gpt-5-chat",
    "requesty:openai/gpt-5-mini",
    "requesty:openai/gpt-5-mini:flex",
    "requesty:openai/gpt-5-mini:priority",
    "requesty:openai/gpt-5-nano",
    "requesty:openai/gpt-5-nano:flex",
    "requesty:openai/gpt-5.1",
    "requesty:openai/gpt-5:flex",
    "requesty:openai/gpt-5:priority",
    "requesty:openai/o1",
    "requesty:openai/o1-pro",
    "requesty:openai/o1:high",
    "requesty:openai/o1:low",
    "requesty:openai/o1:medium",
    "requesty:openai/o3",
    "requesty:openai/o3-deep-research",
    "requesty:openai/o3-mini",
    "requesty:openai/o3-mini:high",
    "requesty:openai/o3-mini:low",
    "requesty:openai/o3-mini:medium",
    "requesty:openai/o3:flex",
    "requesty:openai/o4-mini",
    "requesty:openai/o4-mini-deep-research",
    "requesty:openai/o4-mini:flex",
    "requesty:openai/o4-mini:high",
    "requesty:openai/o4-mini:low",
    "requesty:openai/o4-mini:medium",
    "requesty:parasail/parasail-deepseek-r1",
    "requesty:parasail/parasail-deepseek-v3-0324",
    "requesty:parasail/parasail-gemma3-27b-it",
    "requesty:parasail/parasail-glm-45",
    "requesty:parasail/parasail-kimi-k2-instruct",
    "requesty:parasail/parasail-qwen25-vl-72b-instruct",
    "requesty:parasail/parasail-qwen3-235b-a22b-instruct-2507",
    "requesty:perplexity/sonar",
    "requesty:perplexity/sonar-pro",
    "requesty:perplexity/sonar-reasoning-pro",
    "requesty:smart/task",
    "requesty:together/Qwen/QwQ-32B-Preview",
    "requesty:together/Qwen/Qwen2.5-72B-Instruct-Turbo",
    "requesty:together/Qwen/Qwen2.5-7B-Instruct-Turbo",
    "requesty:together/Qwen/Qwen2.5-Coder-32B-Instruct",
    "requesty:together/deepseek-ai/DeepSeek-R1",
    "requesty:together/deepseek-ai/DeepSeek-V3",
    "requesty:together/meta-llama/Llama-3-70b-chat-hf",
    "requesty:together/meta-llama/Llama-3.2-3B-Instruct-Turbo",
    "requesty:together/meta-llama/Llama-3.3-70B-Instruct-Turbo",
    "requesty:together/meta-llama/LlamaGuard-2-8b",
    "requesty:together/meta-llama/Meta-Llama-3-70B-Instruct-Turbo",
    "requesty:together/meta-llama/Meta-Llama-3-8B-Instruct-Lite",
    "requesty:together/meta-llama/Meta-Llama-3.1-70B-Instruct-Turbo",
    "requesty:together/meta-llama/Meta-Llama-3.1-8B-Instruct-Turbo",
    "requesty:together/meta-llama/Meta-Llama-Guard-3-8B",
    "requesty:vertex/claude-3-5-sonnet",
    "requesty:vertex/claude-3-5-sonnet@europe-west1",
    "requesty:vertex/claude-3-5-sonnet@us-east5",
    "requesty:vertex/claude-3-7-sonnet",
    "requesty:vertex/claude-3-7-sonnet@europe-west1",
    "requesty:vertex/claude-3-7-sonnet@us-east5",
    "requesty:vertex/claude-haiku-4-5",
    "requesty:vertex/claude-haiku-4-5@europe-west1",
    "requesty:vertex/claude-haiku-4-5@us-east5",
    "requesty:vertex/claude-opus-4",
    "requesty:vertex/claude-opus-4-1",
    "requesty:vertex/claude-opus-4-1@us-east5",
    "requesty:vertex/claude-opus-4@us-east5",
    "requesty:vertex/claude-sonnet-4",
    "requesty:vertex/claude-sonnet-4-5",
    "requesty:vertex/claude-sonnet-4-5@europe-west1",
    "requesty:vertex/claude-sonnet-4-5@us-east5",
    "requesty:vertex/claude-sonnet-4@europe-west1",
    "requesty:vertex/claude-sonnet-4@us-east5",
    "requesty:vertex/google/gemini-2.5-flash",
    "requesty:vertex/google/gemini-2.5-flash-image-preview",
    "requesty:vertex/google/gemini-2.5-flash@europe-central2",
    "requesty:vertex/google/gemini-2.5-flash@europe-north1",
    "requesty:vertex/google/gemini-2.5-flash@europe-west1",
    "requesty:vertex/google/gemini-2.5-flash@europe-west4",
    "requesty:vertex/google/gemini-2.5-flash@europe-west8",
    "requesty:vertex/google/gemini-2.5-flash@us-central1",
    "requesty:vertex/google/gemini-2.5-flash@us-east1",
    "requesty:vertex/google/gemini-2.5-flash@us-east5",
    "requesty:vertex/google/gemini-2.5-flash@us-south1",
    "requesty:vertex/google/gemini-2.5-flash@us-west1",
    "requesty:vertex/google/gemini-2.5-pro",
    "requesty:vertex/google/gemini-2.5-pro@europe-central2",
    "requesty:vertex/google/gemini-2.5-pro@europe-north1",
    "requesty:vertex/google/gemini-2.5-pro@europe-west1",
    "requesty:vertex/google/gemini-2.5-pro@europe-west4",
    "requesty:vertex/google/gemini-2.5-pro@europe-west8",
    "requesty:vertex/google/gemini-2.5-pro@us-central1",
    "requesty:vertex/google/gemini-2.5-pro@us-east1",
    "requesty:vertex/google/gemini-2.5-pro@us-east5",
    "requesty:vertex/google/gemini-2.5-pro@us-south1",
    "requesty:vertex/google/gemini-2.5-pro@us-west1",
    "requesty:xai/grok-2-1212",
    "requesty:xai/grok-3",
    "requesty:xai/grok-3-mini",
    "requesty:xai/grok-3-mini:high",
    "requesty:xai/grok-3-mini:low",
    "requesty:xai/grok-4",
    "requesty:xai/grok-4-fast",
    "requesty:xai/grok-4-fast-non-reasoning",
    "requesty:xai/grok-code-fast-1",
    "requesty:zai/GLM-4.5",
    "requesty:zai/GLM-4.6",
]

__all__ = ["RequestyModels"]



"""Models.dev provider package."""

from tokonomics.model_discovery.modelsdev_provider.provider import (
    ModelsDevProvider,
    ModelsDevProviderType,
)

__all__ = ["ModelsDevProvider", "ModelsDevProviderType"]



"""Generated model names for vercel-gateway provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 125
"""

from __future__ import annotations

from typing import Literal


Vercel_GatewayModels = Literal[
    "vercel-gateway:alibaba/qwen-3-14b",
    "vercel-gateway:alibaba/qwen-3-235b",
    "vercel-gateway:alibaba/qwen-3-30b",
    "vercel-gateway:alibaba/qwen-3-32b",
    "vercel-gateway:alibaba/qwen3-coder",
    "vercel-gateway:alibaba/qwen3-coder-30b-a3b",
    "vercel-gateway:alibaba/qwen3-coder-plus",
    "vercel-gateway:alibaba/qwen3-max",
    "vercel-gateway:alibaba/qwen3-max-preview",
    "vercel-gateway:alibaba/qwen3-next-80b-a3b-instruct",
    "vercel-gateway:alibaba/qwen3-next-80b-a3b-thinking",
    "vercel-gateway:alibaba/qwen3-vl-instruct",
    "vercel-gateway:alibaba/qwen3-vl-thinking",
    "vercel-gateway:amazon/nova-lite",
    "vercel-gateway:amazon/nova-micro",
    "vercel-gateway:amazon/nova-pro",
    "vercel-gateway:anthropic/claude-3-haiku",
    "vercel-gateway:anthropic/claude-3-opus",
    "vercel-gateway:anthropic/claude-3.5-haiku",
    "vercel-gateway:anthropic/claude-3.5-sonnet",
    "vercel-gateway:anthropic/claude-3.5-sonnet-20240620",
    "vercel-gateway:anthropic/claude-3.7-sonnet",
    "vercel-gateway:anthropic/claude-haiku-4.5",
    "vercel-gateway:anthropic/claude-opus-4",
    "vercel-gateway:anthropic/claude-opus-4.1",
    "vercel-gateway:anthropic/claude-sonnet-4",
    "vercel-gateway:anthropic/claude-sonnet-4.5",
    "vercel-gateway:cohere/command-a",
    "vercel-gateway:deepseek/deepseek-r1",
    "vercel-gateway:deepseek/deepseek-v3",
    "vercel-gateway:deepseek/deepseek-v3.1",
    "vercel-gateway:deepseek/deepseek-v3.1-terminus",
    "vercel-gateway:deepseek/deepseek-v3.2-exp",
    "vercel-gateway:deepseek/deepseek-v3.2-exp-thinking",
    "vercel-gateway:google/gemini-2.0-flash",
    "vercel-gateway:google/gemini-2.0-flash-lite",
    "vercel-gateway:google/gemini-2.5-flash",
    "vercel-gateway:google/gemini-2.5-flash-image",
    "vercel-gateway:google/gemini-2.5-flash-image-preview",
    "vercel-gateway:google/gemini-2.5-flash-lite",
    "vercel-gateway:google/gemini-2.5-flash-lite-preview-09-2025",
    "vercel-gateway:google/gemini-2.5-flash-preview-09-2025",
    "vercel-gateway:google/gemini-2.5-pro",
    "vercel-gateway:inception/mercury-coder-small",
    "vercel-gateway:meituan/longcat-flash-chat",
    "vercel-gateway:meituan/longcat-flash-thinking",
    "vercel-gateway:meta/llama-3.1-70b",
    "vercel-gateway:meta/llama-3.1-8b",
    "vercel-gateway:meta/llama-3.2-11b",
    "vercel-gateway:meta/llama-3.2-1b",
    "vercel-gateway:meta/llama-3.2-3b",
    "vercel-gateway:meta/llama-3.2-90b",
    "vercel-gateway:meta/llama-3.3-70b",
    "vercel-gateway:meta/llama-4-maverick",
    "vercel-gateway:meta/llama-4-scout",
    "vercel-gateway:minimax/minimax-m2",
    "vercel-gateway:mistral/codestral",
    "vercel-gateway:mistral/devstral-small",
    "vercel-gateway:mistral/magistral-medium",
    "vercel-gateway:mistral/magistral-medium-2506",
    "vercel-gateway:mistral/magistral-small",
    "vercel-gateway:mistral/magistral-small-2506",
    "vercel-gateway:mistral/ministral-3b",
    "vercel-gateway:mistral/ministral-8b",
    "vercel-gateway:mistral/mistral-large",
    "vercel-gateway:mistral/mistral-medium",
    "vercel-gateway:mistral/mistral-small",
    "vercel-gateway:mistral/mixtral-8x22b-instruct",
    "vercel-gateway:mistral/pixtral-12b",
    "vercel-gateway:mistral/pixtral-large",
    "vercel-gateway:moonshotai/kimi-k2",
    "vercel-gateway:moonshotai/kimi-k2-0905",
    "vercel-gateway:moonshotai/kimi-k2-thinking",
    "vercel-gateway:moonshotai/kimi-k2-thinking-turbo",
    "vercel-gateway:moonshotai/kimi-k2-turbo",
    "vercel-gateway:morph/morph-v3-fast",
    "vercel-gateway:morph/morph-v3-large",
    "vercel-gateway:openai/gpt-3.5-turbo",
    "vercel-gateway:openai/gpt-3.5-turbo-instruct",
    "vercel-gateway:openai/gpt-4-turbo",
    "vercel-gateway:openai/gpt-4.1",
    "vercel-gateway:openai/gpt-4.1-mini",
    "vercel-gateway:openai/gpt-4.1-nano",
    "vercel-gateway:openai/gpt-4o",
    "vercel-gateway:openai/gpt-4o-mini",
    "vercel-gateway:openai/gpt-5",
    "vercel-gateway:openai/gpt-5-chat",
    "vercel-gateway:openai/gpt-5-codex",
    "vercel-gateway:openai/gpt-5-mini",
    "vercel-gateway:openai/gpt-5-nano",
    "vercel-gateway:openai/gpt-5-pro",
    "vercel-gateway:openai/gpt-5.1-codex",
    "vercel-gateway:openai/gpt-5.1-codex-mini",
    "vercel-gateway:openai/gpt-5.1-instant",
    "vercel-gateway:openai/gpt-5.1-thinking",
    "vercel-gateway:openai/gpt-oss-120b",
    "vercel-gateway:openai/gpt-oss-20b",
    "vercel-gateway:openai/gpt-oss-safeguard-20b",
    "vercel-gateway:openai/o1",
    "vercel-gateway:openai/o3",
    "vercel-gateway:openai/o3-deep-research",
    "vercel-gateway:openai/o3-mini",
    "vercel-gateway:openai/o4-mini",
    "vercel-gateway:perplexity/sonar",
    "vercel-gateway:perplexity/sonar-pro",
    "vercel-gateway:perplexity/sonar-reasoning",
    "vercel-gateway:perplexity/sonar-reasoning-pro",
    "vercel-gateway:stealth/sonoma-dusk-alpha",
    "vercel-gateway:stealth/sonoma-sky-alpha",
    "vercel-gateway:vercel/v0-1.0-md",
    "vercel-gateway:vercel/v0-1.5-md",
    "vercel-gateway:xai/grok-2",
    "vercel-gateway:xai/grok-2-vision",
    "vercel-gateway:xai/grok-3",
    "vercel-gateway:xai/grok-3-fast",
    "vercel-gateway:xai/grok-3-mini",
    "vercel-gateway:xai/grok-3-mini-fast",
    "vercel-gateway:xai/grok-4",
    "vercel-gateway:xai/grok-4-fast-non-reasoning",
    "vercel-gateway:xai/grok-4-fast-reasoning",
    "vercel-gateway:xai/grok-code-fast-1",
    "vercel-gateway:zai/glm-4.5",
    "vercel-gateway:zai/glm-4.5-air",
    "vercel-gateway:zai/glm-4.5v",
    "vercel-gateway:zai/glm-4.6",
]

__all__ = ["Vercel_GatewayModels"]



"""Generated model names for deepseek provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 2
"""

from __future__ import annotations

from typing import Literal


DeepseekModels = Literal[
    "deepseek:deepseek-chat",
    "deepseek:deepseek-reasoner",
]

__all__ = ["DeepseekModels"]



"""Models.dev provider - unified model discovery from models.dev API."""

from __future__ import annotations

import logging
import os
from typing import TYPE_CHECKING, Any, Literal

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo, ModelPricing


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


logger = logging.getLogger(__name__)


ModelsDevProviderType = Literal[
    "alibaba",
    "alibaba-cn",
    "amazon-bedrock",
    "anthropic",
    "azure",
    "baseten",
    "cerebras",
    "chutes",
    "cloudflare-workers-ai",
    "cohere",
    "cortecs",
    "deepinfra",
    "deepseek",
    "fastrouter",
    "fireworks-ai",
    "github-copilot",
    "github-models",
    "google",
    "google-vertex",
    "google-vertex-anthropic",
    "groq",
    "huggingface",
    "inception",
    "inference",
    "llama",
    "lmstudio",
    "lucidquery",
    "mistral",
    "modelscope",
    "moonshotai",
    "moonshotai-cn",
    "morph",
    "nvidia",
    "openai",
    "opencode",
    "openrouter",
    "perplexity",
    "requesty",
    "submodel",
    "synthetic",
    "togetherai",
    "upstage",
    "v0",
    "venice",
    "vercel",
    "wandb",
    "xai",
    "zai",
    "zai-coding-plan",
    "zhipuai",
    "zhipuai-coding-plan",
]


class ModelsDevProvider(ModelProvider):
    """Models.dev API provider - aggregates models from all providers."""

    def __init__(self, provider: ModelsDevProviderType | None = None):
        super().__init__()
        self.base_url = "https://models.dev"
        self.headers = {}
        self.params = {}
        self.provider_filter = provider

    def is_available(self) -> bool:  # noqa: PLR0911
        """Check whether the provider is available for use."""
        if self.provider_filter == "anthropic":
            return bool(os.environ.get("ANTHROPIC_API_KEY"))
        if self.provider_filter == "openai":
            return bool(os.environ.get("OPENAI_API_KEY"))
        if self.provider_filter == "cohere":
            return bool(os.environ.get("COHERE_API_KEY") or os.environ.get("CO_API_KEY"))
        if self.provider_filter == "cerebras":
            return bool(os.environ.get("CEREBRAS_API_KEY"))
        if self.provider_filter == "deepseek":
            return bool(os.environ.get("DEEPSEEK_API_KEY"))
        if self.provider_filter == "groq":
            return bool(os.environ.get("GROQ_API_KEY"))
        if self.provider_filter == "chutes":
            return bool(os.environ.get("CHUTES_API_KEY"))
        if self.provider_filter == "cloudflare-workers-ai":
            return bool(os.environ.get("CF_API_TOKEN"))
        if self.provider_filter == "cortecs":
            return bool(os.environ.get("CORTECS_API_KEY"))
        if self.provider_filter == "xai":
            return bool(os.environ.get("XAI_API_KEY"))
        if self.provider_filter == "azure":
            return bool(os.environ.get("AZURE_OPENAI_API_KEY"))
        if self.provider_filter == "fireworks-ai":
            return bool(os.environ.get("FIREWORKS_API_KEY"))
        if self.provider_filter == "mistral":
            return bool(os.environ.get("MISTRAL_API_KEY"))
        return True

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse models.dev API response into ModelInfo."""
        # Extract provider from context (set during parsing)
        provider_id = data.get("_provider_id", "unknown")

        # Extract pricing information
        pricing = None
        if "cost" in data:
            cost = data["cost"]
            pricing = ModelPricing(
                prompt=cost.get("input", 0) / 1_000_000 if "input" in cost else None,
                completion=cost.get("output", 0) / 1_000_000
                if "output" in cost
                else None,
                input_cache_read=cost.get("cache_read", 0) / 1_000_000
                if "cache_read" in cost
                else None,
                input_cache_write=cost.get("cache_write", 0) / 1_000_000
                if "cache_write" in cost
                else None,
            )

        # Extract modalities
        input_modalities: set[Modality] = {"text"}
        output_modalities: set[Modality] = {"text"}
        if "modalities" in data:
            modalities = data["modalities"]
            input_modalities = set(modalities.get("input", ["text"]))
            output_modalities = set(modalities.get("output", ["text"]))

        model_id = str(data["id"])

        # Determine if it's an embedding model (heuristic)
        is_embedding = "embedding" in model_id.lower() or "embed" in model_id.lower()

        return ModelInfo(
            id=model_id,
            name=str(data.get("name", model_id)),
            provider=provider_id,
            description=None,  # models.dev doesn't provide descriptions
            pricing=pricing,
            context_window=data.get("limit", {}).get("context"),
            max_output_tokens=data.get("limit", {}).get("output"),
            is_embedding=is_embedding,
            input_modalities=input_modalities,
            output_modalities=output_modalities,
            is_free=pricing is not None
            and pricing.prompt == 0
            and pricing.completion == 0,
            metadata={
                "attachment": data.get("attachment", False),
                "reasoning": data.get("reasoning", False),
                "temperature": data.get("temperature", True),
                "tool_call": data.get("tool_call", False),
                "knowledge": data.get("knowledge"),
                "release_date": data.get("release_date"),
                "last_updated": data.get("last_updated"),
                "open_weights": data.get("open_weights", False),
            },
        )

    async def get_models(self) -> list[ModelInfo]:
        """Fetch all models from models.dev API."""
        from anyenv import HttpError, get_json

        logger.debug("Fetching models from models.dev API")
        data = await get_json(
            f"{self.base_url}/api.json",
            headers=self.headers,
            params=self.params,
            cache=True,
            return_type=dict,
        )

        if not isinstance(data, dict):
            msg = "Invalid response format from models.dev"
            raise RuntimeError(msg)  # noqa: TRY004

        try:
            models = []
            for provider_id, provider_data in data.items():
                # Apply provider filter if specified
                if (
                    self.provider_filter is not None
                    and provider_id != self.provider_filter
                ):
                    continue

                if not isinstance(provider_data, dict) or "models" not in provider_data:
                    continue

                provider_models = provider_data["models"]
                if not isinstance(provider_models, dict):
                    continue

                for model_id, model_info in provider_models.items():
                    if not isinstance(model_info, dict):
                        continue

                    # Add provider context to model data
                    model_data = dict(model_info)
                    model_data["id"] = model_id
                    model_data["_provider_id"] = provider_id

                    try:
                        model = self._parse_model(model_data)
                        models.append(model)
                    except Exception as e:  # noqa: BLE001
                        logger.warning(
                            "Failed to parse model %s from provider %s: %s",
                            model_id,
                            provider_id,
                            e,
                        )
                        continue

            if self.provider_filter:
                logger.debug(
                    "Fetched %d models for provider %s via models.dev",
                    len(models),
                    self.provider_filter,
                )
            else:
                logger.info(
                    "Fetched %d models from %d providers via models.dev",
                    len(models),
                    len([
                        p
                        for p in data
                        if isinstance(data[p], dict) and "models" in data[p]
                    ]),
                )
        except HttpError as e:
            msg = f"Failed to fetch models from models.dev: {e}"
            raise RuntimeError(msg) from e
        else:
            return models


if __name__ == "__main__":
    import asyncio

    async def main():
        # Test all providers
        provider = ModelsDevProvider()
        models = await provider.get_models()
        print(f"All providers: {len(models)} models")

        # Test specific provider
        openai_provider = ModelsDevProvider(provider="openai")
        openai_models = await openai_provider.get_models()
        print(f"OpenAI only: {len(openai_models)} models")

        # Test another specific provider
        anthropic_provider = ModelsDevProvider(provider="anthropic")
        anthropic_models = await anthropic_provider.get_models()
        print(f"Anthropic only: {len(anthropic_models)} models")

        # Show sample models
        if openai_models:
            print("\nSample OpenAI model:")
            print(openai_models[0].format())

    asyncio.run(main())



"""xAI provider."""

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo, ModelPricing


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


class XAIProvider(ModelProvider):
    """xAI API provider."""

    def __init__(self, api_key: str | None = None):
        super().__init__()
        self.api_key = api_key or os.environ.get("XAI_API_KEY")
        if not self.api_key:
            msg = "xAI API key not found in parameters or XAI_API_KEY env var"
            raise RuntimeError(msg)

        self.base_url = "https://api.x.ai/v1"
        self.headers = {"Authorization": f"Bearer {self.api_key}"}
        self.params = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.api_key)

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse xAI API response into ModelInfo."""
        # Extract basic info
        model_id = str(data["id"])
        name = str(data.get("name", model_id))
        owned_by = str(data.get("owned_by"))

        # Parse modalities
        input_modalities: set[Modality] = set()
        output_modalities: set[Modality] = set()

        for modality in data.get("input_modalities", []):
            if modality in {"text", "image", "audio", "video"}:
                input_modalities.add(modality)

        for modality in data.get("output_modalities", []):
            if modality in {"text", "image", "audio", "video"}:
                output_modalities.add(modality)

        # Set defaults if empty
        if not input_modalities:
            input_modalities = {"text"}
        if not output_modalities:
            output_modalities = {"text"}

        # Parse pricing (prices appear to be in micro-units, convert to dollars per token)
        pricing = None
        if any(
            key in data
            for key in [
                "prompt_text_token_price",
                "completion_text_token_price",
                "prompt_image_token_price",
                "search_price",
            ]
        ):
            pricing = ModelPricing(
                prompt=data.get("prompt_text_token_price", 0) / 1_000_000
                if "prompt_text_token_price" in data
                else None,
                completion=data.get("completion_text_token_price", 0) / 1_000_000
                if "completion_text_token_price" in data
                else None,
                image=data.get("prompt_image_token_price", 0) / 1_000_000
                if "prompt_image_token_price" in data
                else None,
                input_cache_read=data.get("cached_prompt_text_token_price", 0) / 1_000_000
                if "cached_prompt_text_token_price" in data
                else None,
                web_search=data.get("search_price", 0) / 1_000_000
                if "search_price" in data
                else None,
            )

        # Build metadata
        metadata = {}
        if "fingerprint" in data:
            metadata["fingerprint"] = data["fingerprint"]
        if "version" in data:
            metadata["version"] = data["version"]
        if "created" in data:
            metadata["created_timestamp"] = data["created"]
        if data.get("aliases"):
            metadata["aliases"] = data["aliases"]

        return ModelInfo(
            id=model_id,
            name=name,
            provider="xai",
            owned_by=owned_by,
            pricing=pricing,
            input_modalities=input_modalities,
            output_modalities=output_modalities,
            metadata=metadata,
        )

    async def get_models(self) -> list[ModelInfo]:
        """Override the base method to handle xAI's unique API structure."""
        from anyenv import HttpError, get_json

        url = f"{self.base_url}/language-models"

        try:
            data = await get_json(
                url,
                headers=self.headers,
                params=self.params,
                cache=True,
                return_type=dict,
            )

            if "models" not in data:
                msg = "Invalid response format from xAI API"
                raise RuntimeError(msg)

            return [self._parse_model(item) for item in data["models"]]

        except HttpError as e:
            msg = f"Failed to fetch models from xAI: {e}"
            raise RuntimeError(msg) from e


if __name__ == "__main__":
    import asyncio

    async def main():
        provider = XAIProvider()
        models = await provider.get_models()
        for model in models:
            print(model.format())

    asyncio.run(main())



"""Generated model names for groq provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 20
"""

from __future__ import annotations

from typing import Literal


GroqModels = Literal[
    "groq:allam-2-7b",
    "groq:groq/compound",
    "groq:groq/compound-mini",
    "groq:llama-3.1-8b-instant",
    "groq:llama-3.3-70b-versatile",
    "groq:meta-llama/llama-4-maverick-17b-128e-instruct",
    "groq:meta-llama/llama-4-scout-17b-16e-instruct",
    "groq:meta-llama/llama-guard-4-12b",
    "groq:meta-llama/llama-prompt-guard-2-22m",
    "groq:meta-llama/llama-prompt-guard-2-86m",
    "groq:moonshotai/kimi-k2-instruct",
    "groq:moonshotai/kimi-k2-instruct-0905",
    "groq:openai/gpt-oss-120b",
    "groq:openai/gpt-oss-20b",
    "groq:openai/gpt-oss-safeguard-20b",
    "groq:playai-tts",
    "groq:playai-tts-arabic",
    "groq:qwen/qwen3-32b",
    "groq:whisper-large-v3",
    "groq:whisper-large-v3-turbo",
]

__all__ = ["GroqModels"]



"""GitHub Copilot provider for model discovery."""

from __future__ import annotations

from datetime import datetime
import logging
from typing import TYPE_CHECKING, Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.copilot_provider.token_manager import token_manager
from tokonomics.model_discovery.model_info import ModelInfo


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


logger = logging.getLogger(__name__)

# Models to exclude from the provider
COPILOT_EXCLUDED_MODELS = set[str]()  # "o1"


class CopilotProvider(ModelProvider):
    """GitHub Copilot API provider."""

    def __init__(self):
        super().__init__()
        self._token_manager = token_manager
        self.base_url = self._token_manager._api_endpoint
        self.params = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return self._token_manager.is_available()

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse Copilot API response into ModelInfo."""
        # Extract capabilities and limits
        capabilities = data.get("capabilities", {})
        limits = capabilities.get("limits", {})

        # Extract context window, input and output tokens
        context_window = limits.get("max_context_window_tokens")
        _max_input_tokens = limits.get("max_input_tokens")
        max_output_tokens = limits.get("max_output_tokens")

        # Determine modalities
        input_modalities: set[Modality] = {"text"}
        output_modalities: set[Modality] = {"text"}

        # Add vision capability if supported
        if capabilities.get("supports", {}).get("vision"):
            input_modalities.add("image")

        # Extract model family and version info
        model_family = capabilities.get("family", "")
        model_version = data.get("version", "")

        # Create description
        description = ""
        if model_family:
            description += f"Model family: {model_family}\n"
        if model_version:
            description += f"Version: {model_version}\n"
        if vendor := data.get("vendor"):
            description += f"Vendor: {vendor}\n"

        # Add capabilities to description
        supports = capabilities.get("supports", {})
        support_features = []
        if supports.get("tool_calls"):
            support_features.append("tool calls")
        if supports.get("parallel_tool_calls"):
            support_features.append("parallel tool calls")
        if supports.get("vision"):
            support_features.append("vision")
        if supports.get("streaming"):
            support_features.append("streaming")

        if support_features:
            description += f"Supports: {', '.join(support_features)}"

        return ModelInfo(
            id=str(data["id"]),
            name=str(data.get("name", data["id"])),
            provider="copilot",
            description=description.strip() or None,
            context_window=context_window,
            max_output_tokens=max_output_tokens,
            is_deprecated=not data.get("model_picker_enabled", True),
            owned_by=data.get("vendor"),
            input_modalities=input_modalities,
            output_modalities=output_modalities,
        )

    async def get_models(self) -> list[ModelInfo]:
        """Override the standard get_models to use Copilot's specific endpoint."""
        import anyenv

        try:
            headers = await self._token_manager.generate_headers()
            url = f"{self.base_url}/models"
            response = await anyenv.get(url, headers=headers, timeout=30)
            data = await response.json()
            if not isinstance(data, dict) or "data" not in data:
                msg = f"Invalid response format from Copilot API: {data}"
                raise RuntimeError(msg)  # noqa: TRY301
            models = []
            for model in data["data"]:
                # Skip models without model picker enabled
                if not model.get("model_picker_enabled", False):
                    continue
                capabilities = model.get("capabilities", {})
                if (
                    capabilities.get("type") != "chat"
                    or not capabilities.get("supports", {}).get("tool_calls", False)
                    or model["id"] in COPILOT_EXCLUDED_MODELS
                ):
                    continue

                models.append(self._parse_model(model))
        except Exception as e:
            msg = f"Failed to fetch models from Copilot: {e}"
            logger.exception(msg)
            raise RuntimeError(msg) from e
        else:
            return models


if __name__ == "__main__":
    import anyenv

    # Set up logging
    logging.basicConfig(
        level=logging.DEBUG, format="%(asctime)s - %(name)s - %(levelname)s - %(message)s"
    )

    async def test_provider():
        try:
            # Create provider
            provider = CopilotProvider()
            print(f"Provider initialized: {provider.base_url}")
            print("\nFetching models asynchronously...")
            models = await provider.get_models()
            print(f"Found {len(models)} models from Copilot API:")
            for i, model in enumerate(models, 1):
                print(f"\n{i}. {model.name} ({model.id})")
                print(f"   Context window: {model.context_window}")
            print("\nTesting token refresh mechanism...")
            # Force token refresh by setting expiry to now
            provider._token_manager._token_expires_at = datetime.now()
            models = await provider.get_models()
            print("Token refresh successful, found models:", len(models))

        except Exception as e:  # noqa: BLE001
            print(f"Error: {e}")

    # Run the async test function
    anyenv.run_sync(test_provider())



"""Utilities for calculating token costs using LiteLLM pricing data."""

from __future__ import annotations

from decimal import Decimal
import os
import pathlib
from typing import Any, cast

from anyenv import get_json
from platformdirs import user_data_dir

from tokonomics import log
from tokonomics.helpers import _is_numeric, _safe_numeric_convert
from tokonomics.toko_types import ModelCapabilities, ModelCosts, TokenCosts


logger = log.get_logger(__name__)


# Cache cost data persistently
PRICING_DIR = pathlib.Path(user_data_dir("tokonomics", "tokonomics")) / "pricing"
PRICING_DIR.mkdir(parents=True, exist_ok=True)
_cost_cache: dict[str, object] = {}

# Cache timeout in seconds (24 hours)
_CACHE_TIMEOUT = 86400


def _is_testing() -> bool:
    """Check if we're running in a test environment."""
    return bool(os.getenv("PYTEST_CURRENT_TEST"))


LITELLM_PRICES_URL = "https://raw.githubusercontent.com/BerriAI/litellm/refs/heads/main/model_prices_and_context_window.json"


def reset_cache() -> None:
    """Clear all cached pricing and limit data."""
    _cost_cache.clear()


def _find_litellm_model_name(model: str) -> str | None:
    """Find matching model name in LiteLLM pricing data.

    Attempts to match the input model name against cached LiteLLM pricing data
    by trying different formats (direct match, base name, provider format).

    Args:
        model: Input model name (e.g. "openai:gpt-4", "gpt-4")

    Returns:
        str | None: Matching LiteLLM model name if found in cache, None otherwise
    """
    logger.debug("Looking up model costs for: %s", model)
    model = model.lower()
    if model in _cost_cache:
        logger.debug("Found direct cache match for: %r", model)
        return model

    if ":" in model:  # For provider:model format, try both variants
        provider, model_name = model.split(":", 1)
        if _cost_cache.get(model_name) is not None:
            logger.debug("Found cache match for base name: %r", model_name)
            return model_name
        # Try provider/model  (normalized)
        if _cost_cache.get(key := f"{provider.lower()}/{model_name}") is not None:
            logger.debug("Found cache match for provider format: %r", key)
            return key
    logger.debug("No cache match found for: %r", model)
    return None


async def get_model_costs(
    model: str,
    *,
    cache_timeout: int = _CACHE_TIMEOUT,
) -> ModelCosts | None:
    """Get cost information for a model from LiteLLM pricing data.

    Attempts to find model costs in cache first. If not found, downloads fresh
    pricing data from LiteLLM's GitHub repository and updates the cache.

    Args:
        model: Name of the model to look up costs for
        cache_timeout: Number of seconds to keep prices in cache (default: 24 hours)

    Returns:
        ModelCosts | None: Model's cost information if found, None otherwise
    """
    normalized_model = model.lower()
    cache_key = f"{normalized_model}_costs"
    cached_costs = cast(ModelCosts | None, _cost_cache.get(cache_key))
    if cached_costs is not None:
        return cached_costs

    try:
        logger.debug("Downloading pricing data from LiteLLM...")
        data: dict[str, Any] = await get_json(
            LITELLM_PRICES_URL,
            cache=not _is_testing(),
            cache_ttl=_CACHE_TIMEOUT,
            return_type=dict,
        )
        assert isinstance(data, dict), f"Expected dict, got {type(data)}"
        logger.debug("Successfully downloaded pricing data.")

        all_costs: dict[str, ModelCosts] = {}
        for name, info in data.items():
            if not isinstance(info, dict):  # Skip sample_spec
                continue

            # Get cost values safely
            input_cost = info.get("input_cost_per_token")
            output_cost = info.get("output_cost_per_token")

            # Skip if values are missing
            if input_cost is None or output_cost is None:
                continue

            # Skip if values aren't numeric
            if not (_is_numeric(input_cost) and _is_numeric(output_cost)):
                continue

            # Store with normalized case
            all_costs[name.lower()] = ModelCosts(
                input_cost_per_token=Decimal(str(input_cost)),
                output_cost_per_token=Decimal(str(output_cost)),
            )

        logger.debug("Extracted costs for %d models", len(all_costs))

        # Update cache with all costs
        for model_name, cost_info in all_costs.items():
            cost_cache_key = f"{model_name}_costs"
            _cost_cache[cost_cache_key] = cost_info
        logger.debug("Updated cache with new pricing data")

        # Try finding model with different formats
        result = all_costs.get(normalized_model)
        if result is None and ":" in normalized_model:
            provider, model_name = normalized_model.split(":", 1)
            # Try base model name
            result = all_costs.get(model_name)
            if result is None:
                provider_format = f"{provider}/{model_name}"
                result = all_costs.get(provider_format)

        # Cache the result for this specific model name
        if result is not None:
            _cost_cache[cache_key] = result
    except Exception as e:  # noqa: BLE001
        logger.debug("Failed to get model costs for %r: %s", model, e)
        return None
    else:
        return result


async def calculate_token_cost(
    model: str,
    input_tokens: int | None,
    output_tokens: int | None,
    *,
    cache_timeout: int = _CACHE_TIMEOUT,
) -> TokenCosts | None:
    """Calculate detailed costs for token usage based on model pricing.

    Combines input and output token counts with their respective costs to
    calculate the breakdown of costs. If either token count is None, it will
    be treated as 0 tokens.

    Args:
        model: Name of the model used (e.g. "gpt-4", "openai:gpt-3.5-turbo")
        input_tokens: Number of tokens in the prompt/input, or None
        output_tokens: Number of tokens in the completion/output, or None
        cache_timeout: Number of seconds to keep prices in cache (default: 24 hours)

    Returns:
        TokenCosts | None: Detailed cost breakdown if pricing data available
    """
    costs = await get_model_costs(model, cache_timeout=cache_timeout)
    if not costs:
        logger.debug("No costs found for model %r", model)
        return None

    # Convert None values to 0
    input_count = input_tokens or 0
    output_count = output_tokens or 0

    input_cost = input_count * costs["input_cost_per_token"]
    output_cost = output_count * costs["output_cost_per_token"]

    token_costs = TokenCosts(input_cost=input_cost, output_cost=output_cost)

    logger.debug(
        "Cost calculation - prompt: $%.6f, completion: $%.6f, total: $%.6f",
        token_costs.input_cost,
        token_costs.output_cost,
        token_costs.total_cost,
    )
    return token_costs


async def get_available_models(
    *,
    cache_timeout: int = _CACHE_TIMEOUT,
) -> list[str]:
    """Get a list of all available model names from LiteLLM pricing data.

    Args:
        cache_timeout: Number of seconds to keep data in cache (default: 24 hours)

    Returns:
        list[str]: List of available model names, sorted alphabetically
    """
    cache_key = "available_models"

    cached_models = cast(list[str] | None, _cost_cache.get(cache_key))
    if cached_models is not None:
        return cached_models

    try:
        logger.debug("Downloading model data from LiteLLM...")
        data: dict[str, Any] = await get_json(
            LITELLM_PRICES_URL,
            cache=not _is_testing(),
            cache_ttl=_CACHE_TIMEOUT,
            return_type=dict,
        )
        assert isinstance(data, dict), f"Expected dict, got {type(data)}"
        # Filter out non-dictionary entries (like sample_spec) and collect model names
        model_names = sorted(
            name.lower() for name, info in data.items() if isinstance(info, dict)
        )

        logger.debug("Found %d available models", len(model_names))

        # Cache the results
        _cost_cache[cache_key] = model_names
    except Exception as e:
        error_msg = "Failed to get available models"
        logger.exception(error_msg)
        raise ValueError(error_msg) from e
    else:
        return model_names


async def get_model_capabilities(
    model: str,
    *,
    cache_timeout: int = _CACHE_TIMEOUT,
) -> ModelCapabilities | None:
    """Get capabilities information for a model from LiteLLM data.

    Args:
        model: Name of the model to look up capabilities for
        cache_timeout: Number of seconds to keep data in cache (default: 24 hours)

    Returns:
        ModelCapabilities | None: Model's capabilities if found, None otherwise

    Raises:
        ValueError: If there's an error fetching the capabilities data
    """
    normalized_model = model.lower()
    cache_key = f"{normalized_model}_capabilities"

    cached_capabilities = cast(ModelCapabilities | None, _cost_cache.get(cache_key))
    if cached_capabilities is not None:
        return cached_capabilities

    try:
        logger.debug("Downloading model data from LiteLLM...")
        data: dict[str, Any] = await get_json(
            LITELLM_PRICES_URL,
            cache=not _is_testing(),
            cache_ttl=_CACHE_TIMEOUT,
            return_type=dict,
        )
        assert isinstance(data, dict), f"Expected dict, got {type(data)}"
        logger.debug("Successfully downloaded model data")

        all_capabilities: dict[str, ModelCapabilities] = {}
        for name, info in data.items():
            if not isinstance(info, dict):
                continue

            # Handle token limits with defaults
            max_tokens_raw = info.get("max_tokens", 0)
            max_input_raw = info.get("max_input_tokens", max_tokens_raw)
            max_output_raw = info.get("max_output_tokens", max_tokens_raw)

            # Convert token values to integers, defaulting to 0
            try:
                max_tokens = int(_safe_numeric_convert(max_tokens_raw))
                max_input = int(_safe_numeric_convert(max_input_raw))
                max_output = int(_safe_numeric_convert(max_output_raw))
            except (ValueError, TypeError):
                max_tokens = max_input = max_output = 0

            capabilities = ModelCapabilities(
                max_tokens=max_tokens,
                max_input_tokens=max_input,
                max_output_tokens=max_output,
                litellm_provider=info.get("litellm_provider"),
                mode=info.get("mode"),
                supports_function_calling=bool(info.get("supports_function_calling")),
                supports_parallel_function_calling=bool(
                    info.get("supports_parallel_function_calling")
                ),
                supports_vision=bool(info.get("supports_vision")),
                supports_audio_input=bool(info.get("supports_audio_input")),
                supports_audio_output=bool(info.get("supports_audio_output")),
                supports_prompt_caching=bool(info.get("supports_prompt_caching")),
                supports_response_schema=bool(info.get("supports_response_schema")),
                supports_system_messages=bool(info.get("supports_system_messages")),
            )
            all_capabilities[name.lower()] = capabilities

        logger.debug("Extracted capabilities for %d models", len(all_capabilities))

        # Update cache with all capabilities
        for model_name, model_capabilities in all_capabilities.items():
            cap_cache_key = f"{model_name}_capabilities"
            _cost_cache[cap_cache_key] = model_capabilities
        logger.debug("Updated cache with new capabilities data")

        # Try finding model with different formats
        result = all_capabilities.get(normalized_model)
        if result is None and ":" in normalized_model:
            provider, model_name = normalized_model.split(":", 1)
            # Try base model name
            result = all_capabilities.get(model_name)
            if result is None:
                # Try provider/model format
                provider_format = f"{provider}/{model_name}"
                result = all_capabilities.get(provider_format)

        if result is not None:
            _cost_cache[cache_key] = result
            return result

    except Exception as e:
        error_msg = f"Failed to get model capabilities for {model}: {e}"
        logger.exception(error_msg)
        raise ValueError(error_msg) from e

    return None


if __name__ == "__main__":
    import asyncio

    async def main():
        # Example usage
        model_name = "openrouter:openai/gpt-5-nano"
        limits = await get_model_costs(model_name)
        print(limits)

    asyncio.run(main())



"""Generated model names for moonshotai provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 5
"""

from __future__ import annotations

from typing import Literal


MoonshotaiModels = Literal[
    "moonshotai:kimi-k2-0711-preview",
    "moonshotai:kimi-k2-0905-preview",
    "moonshotai:kimi-k2-thinking",
    "moonshotai:kimi-k2-thinking-turbo",
    "moonshotai:kimi-k2-turbo-preview",
]

__all__ = ["MoonshotaiModels"]



"""DeepSeek provider."""

from __future__ import annotations

import os
from typing import Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo


class DeepSeekProvider(ModelProvider):
    """DeepSeek API provider."""

    def __init__(self, api_key: str | None = None):
        super().__init__()
        self.api_key = api_key or os.environ.get("DEEPSEEK_API_KEY")
        if not self.api_key:
            msg = "DeepSeek API key not found in parameters or DEEPSEEK_API_KEY env var"
            raise RuntimeError(msg)

        self.base_url = "https://api.deepseek.com/v1"
        self.headers = {"Authorization": f"Bearer {self.api_key}"}
        self.params = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.api_key)

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse DeepSeek API response into ModelInfo."""
        return ModelInfo(
            id=str(data["id"]),
            name=str(data.get("name", data["id"])),
            provider="deepseek",
            owned_by=str(data.get("owned_by")),
            description=str(data.get("description")) if "description" in data else None,
            context_window=(
                int(data["context_window"]) if "context_window" in data else None
            ),
        )


if __name__ == "__main__":
    import asyncio

    provider = DeepSeekProvider()
    models = asyncio.run(provider.get_models())
    for model in models:
        print(model.format())



"""Cerebras provider."""

from __future__ import annotations

import logging
import os
from typing import Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo


logger = logging.getLogger(__name__)


class CerebrasProvider(ModelProvider):
    """Cerebras API provider."""

    def __init__(self, api_key: str | None = None):
        super().__init__()
        self.api_key = api_key or os.environ.get("CEREBRAS_API_KEY")
        if not self.api_key:
            msg = "Cerebras API key not found in parameters or CEREBRAS_API_KEY env var"
            raise RuntimeError(msg)

        self.base_url = "https://api.cerebras.ai/v1"
        self.headers = {"Authorization": f"Bearer {self.api_key}"}
        self.params = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.api_key)

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse Cerebras API response into ModelInfo."""
        # Extract model ID and creation time
        model_id = data.get("id", "")
        created_timestamp = data.get("created")

        # Extract owner information
        owned_by = data.get("owned_by")

        # Context window sizes based on known information about Llama models
        # These are estimates and should be updated with accurate info when available
        context_window = None
        if model_id == "llama3.1-8b":
            context_window = 8192  # Example value
        elif model_id == "llama-3.3-70b":
            context_window = 32768  # Example value

        # Create a description that includes creation timestamp if available
        description = None
        if created_timestamp:
            from datetime import datetime

            try:
                created_date = datetime.fromtimestamp(created_timestamp)
                formatted_date = created_date.strftime("%B %d, %Y")
                description = f"Model released: {formatted_date}"
            except (ValueError, TypeError, OverflowError) as e:
                logger.warning("Could not parse timestamp %s: %s", created_timestamp, e)

        return ModelInfo(
            id=model_id,
            name=model_id,
            provider="cerebras",
            description=description,
            owned_by=owned_by,
            context_window=context_window,
        )

    async def get_models(self) -> list[ModelInfo]:
        """Fetch available models from the Cerebras API."""
        from anyenv import HttpError, get_json

        url = f"{self.base_url}/models"

        try:
            data = await get_json(
                url,
                headers=self.headers,
                params=self.params,
                cache=True,
                return_type=dict,
            )

            if not isinstance(data, dict) or "data" not in data:
                msg = "Invalid response format from Cerebras API"
                raise RuntimeError(msg)

            return [self._parse_model(item) for item in data["data"]]

        except HttpError as e:
            msg = f"Failed to fetch models from Cerebras: {e}"
            raise RuntimeError(msg) from e


if __name__ == "__main__":
    import asyncio

    async def main():
        provider = CerebrasProvider()
        models = await provider.get_models()
        for model in models:
            print(model)

    asyncio.run(main())



"""Clean API models for tokonomics."""

from tokonomics.models.chat_model import (
    ModelCapabilities,
    ModelLimits,
    ChatModel,
    ModelPricing,
)

__all__ = [
    "ChatModel",
    "ModelCapabilities",
    "ModelLimits",
    "ModelPricing",
]



from __future__ import annotations


ARTIFICIAL_ANALYSIS = {
    "o1": 90.0,
    "o3-mini": 89.0,
    "o1-preview": 85.0,  # From Microsoft Azure provider
    "o1-mini": 84.0,  # Taking the higher score from Microsoft Azure
    "gpt-4o-2024-08-06": 78.0,
    "gpt-4o-2024-05-13": 78.0,
    "gpt-4o": 75.0,  # Taking the higher score from Microsoft Azure
    "gpt-4o-mini": 73.0,
    "llama-3-3-instruct-70b": 75.0,  # Using the highest score (Together.ai Turbo)
    "llama-3-3-instruct-70b-vision": 68.0,  # Using the highest score
    "llama-3-1-instruct-405b": 75.0,  # Using the highest score (Hyperbolic)
    "llama-3-1-instruct-70b": 69.0,  # Using the highest score (Hyperbolic)
    "llama-3-2-instruct-90b-vision": 68.0,  # Using the highest score (Google Vertex)
    "llama-3-2-instruct-11b-vision": 54.0,  # Using the highest score
    "llama-3-1-instruct-8b": 54.0,  # Using the highest score
    "llama-3-2-instruct-3b": 50.0,  # Using the highest score (Fireworks)
    "llama-3-2-instruct-1b": 26.0,
    "gemini-2-0-flash-experimental": 82.0,
    "gemini-1-5-pro": 80.0,
    "gemini-1-5-flash": 74.0,
    "gemini-1-5-pro-may-2024": 72.0,
    "gemma-2-27b": 61.0,
    "gemma-2-9b": 55.0,
    "gemini-1-5-flash-8b": 47.0,
    "gemini-1-5-flash-may-2024": 60.0,
    "gemini-experimental-dec-2024": 0.0,
    "claude-35-sonnet": 80.0,
    "claude-35-sonnet-june-24": 76.0,
    "claude-3-opus": 70.0,
    "claude-3-5-haiku": 68.0,
    "claude-3-haiku": 55.0,
    "pixtral-large-2411": 74.0,
    "mistral-large-2": 74.0,
    "mistral-large-2407": 74.0,
    "mistral-small-3": 72.0,
    "mistral-small": 61.0,
    "mistral-8x22b-instruct": 62.0,  # Using the highest score (Mistral)
    "pixtral": 57.0,  # Using the highest score (Hyperbolic)
    "ministral-8b": 56.0,
    "mistral-nemo": 54.0,  # Using the highest score (Deepinfra)
    "ministral-3b": 53.0,
    "mixtral-8x7b-instruct": 43.0,  # Using the highest score (Fireworks)
    "codestral-mamba": 33.0,
    "codestral": 0.0,
    "command-r-plus": 55.0,
    "command-r-plus-04-2024": 47.0,  # Using the highest score (Cohere)
    "command-r-03-2024": 37.0,  # Using the highest score (Cohere)
    "command-r": 51.0,
    "aya-expanse-32b": 0.0,
    "aya-expanse-8b": 0.0,
    "grok-beta": 72.0,
    "nova-pro": 75.0,
    "nova-lite": 70.0,
    "nova-micro": 65.0,
    "phi-4": 76.0,
    "phi-3-medium": 0.0,
    "dbrx": 50.0,  # Using the highest score (Databricks)
    "llama-3-1-nemotron-instruct-70b": 72.0,
    "jamba-1-5-large": 64.0,
    "jamba-1-5-mini": 46.0,
    "deepseek-r1": 89.0,
    "deepseek-v3": 80.0,  # Using the highest score (DeepSeek)
    "deepseek-llm-67b-chat": 47.0,
    "deepseek-r1-distill-llama-70b": 0.0,
    "deepseek-v2-5-sep-2024": 0.0,
    "qwen-2-5-max": 79.0,
    "qwen2-5-72b-instruct": 78.0,  # Using the highest score (Deepinfra)
    "qwen2-5-coder-32b-instruct": 72.0,
    "qwen-turbo": 71.0,
    "qwen2-72b-instruct": 68.0,
    "QwQ-32B-Preview": 0.0,
    "qwen2.5-32b-instruct": 0.0,
    "yi-large": 62.0,
    "gpt-4-turbo": 75.0,
    "gpt-4": 0.0,
    "llama-3-instruct-70b": 62.0,  # Using the highest scores (Hyperbolic/Novita)
    "llama-3-instruct-8b": 53.0,  # Using the highest score (Novita)
    "llama-2-chat-7b": 0.0,
    "gemini-pro": 0.0,
    "claude-3-sonnet": 57.0,
    "claude-21": 0.0,
    "claude-2": 0.0,
    "mistral-small-2402": 59.0,
    "mistral-large": 57.0,  # Using the highest score (Mistral)
    "mistral-7b-instruct": 33.0,  # Using the highest score (Novita)
    "codestral-2405": 0.0,
    "mistral-medium": 0.0,
    "sonar-3-1-small-chat": 0.0,
    "sonar-3-1-large-chat": 0.0,
    "openchat-35": 44.0,
    "jamba-instruct": 28.0,
}



"""Generated model names for anthropic provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 19
"""

from __future__ import annotations

from typing import Literal


AnthropicModels = Literal[
    "anthropic:claude-3-5-haiku-20241022",
    "anthropic:claude-3-5-haiku-latest",
    "anthropic:claude-3-5-sonnet-20240620",
    "anthropic:claude-3-5-sonnet-20241022",
    "anthropic:claude-3-7-sonnet-20250219",
    "anthropic:claude-3-7-sonnet-latest",
    "anthropic:claude-3-haiku-20240307",
    "anthropic:claude-3-opus-20240229",
    "anthropic:claude-3-sonnet-20240229",
    "anthropic:claude-haiku-4-5",
    "anthropic:claude-haiku-4-5-20251001",
    "anthropic:claude-opus-4-0",
    "anthropic:claude-opus-4-1",
    "anthropic:claude-opus-4-1-20250805",
    "anthropic:claude-opus-4-20250514",
    "anthropic:claude-sonnet-4-0",
    "anthropic:claude-sonnet-4-20250514",
    "anthropic:claude-sonnet-4-5",
    "anthropic:claude-sonnet-4-5-20250929",
]

__all__ = ["AnthropicModels"]



"""Tokonomics: main package.

Calcuate costs for LLM Usage based on token count.
"""

from __future__ import annotations

from importlib.metadata import version

__version__ = version("tokonomics")
__title__ = "Tokonomics"

__author__ = "Philipp Temminghoff"
__author_email__ = "philipptemminghoff@googlemail.com"
__copyright__ = "Copyright (c) 2024 Philipp Temminghoff"
__license__ = "MIT"
__url__ = "https://github.com/phil65/tokonomics"

from tokonomics.core import (
    get_model_costs,
    calculate_token_cost,
    get_model_capabilities,
    get_available_models,
    reset_cache,
)
from tokonomics.toko_types import ModelCosts, TokenCosts
from tokonomics.pydanticai_cost import calculate_pydantic_cost, Usage
from tokonomics.model_discovery import (
    AnthropicProvider,
    MistralProvider,
    OpenRouterProvider,
    OpenAIProvider,
    GroqProvider,
    ModelInfo,
    ModelPricing,
    ModelProvider,
    get_all_models,
)
from tokonomics.model_discovery.copilot_provider import CopilotTokenManager
from tokonomics.token_count import count_tokens
from tokonomics.model_names import ModelName


__all__ = [
    "AnthropicProvider",
    "CopilotTokenManager",
    "GroqProvider",
    "MistralProvider",
    "ModelCosts",
    "ModelInfo",
    "ModelName",
    "ModelPricing",
    "ModelProvider",
    "OpenAIProvider",
    "OpenRouterProvider",
    "TokenCosts",
    "Usage",
    "__version__",
    "calculate_pydantic_cost",
    "calculate_token_cost",
    "count_tokens",
    "get_all_models",
    "get_available_models",
    "get_model_capabilities",
    "get_model_costs",
    "reset_cache",
]



"""Model discovery package."""

from __future__ import annotations

import asyncio
import logging
from functools import partial
from typing import Literal, TYPE_CHECKING

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelPricing, ModelInfo
from tokonomics.model_discovery.groq_provider import GroqProvider
from tokonomics.model_discovery.mistral_provider import MistralProvider
from tokonomics.model_discovery.openrouter_provider import OpenRouterProvider
from tokonomics.model_discovery.github_provider import GitHubProvider
from tokonomics.model_discovery.copilot_provider import CopilotProvider, token_manager
from tokonomics.model_discovery.gemini_provider import GeminiProvider
from tokonomics.model_discovery.deepseek_provider import DeepSeekProvider
from tokonomics.model_discovery.requesty_provider import RequestyProvider
from tokonomics.model_discovery.xai_provider import XAIProvider
from tokonomics.model_discovery.comet_provider import CometProvider
from tokonomics.model_discovery.novita_provider import NovitaProvider
from tokonomics.model_discovery.vercel_gateway_provider import VercelGatewayProvider
from tokonomics.model_discovery.modelsdev_provider import ModelsDevProvider
from tokonomics.model_discovery.ollama_provider import OllamaProvider
# from tokonomics.model_discovery.cerebras_provider import CerebrasProvider
# from tokonomics.model_discovery.cohere_provider import CohereProvider

# Use ModelsDevProvider with pre-configured filters as drop-in replacements
AnthropicProvider = partial(ModelsDevProvider, provider="anthropic")
OpenAIProvider = partial(ModelsDevProvider, provider="openai")
CohereProvider = partial(ModelsDevProvider, provider="cohere")
CerebrasProvider = partial(ModelsDevProvider, provider="cerebras")
FireworksProvider = partial(ModelsDevProvider, provider="fireworks-ai")
AzureProvider = partial(ModelsDevProvider, provider="azure")
ChutesProvider = partial(ModelsDevProvider, provider="chutes")
CortecsProvider = partial(ModelsDevProvider, provider="cortecs")
MoonshotProvider = partial(ModelsDevProvider, provider="moonshotai")
ZAIProvider = partial(ModelsDevProvider, provider="zai")


if TYPE_CHECKING:
    from collections.abc import Callable
    from collections.abc import Sequence


logger = logging.getLogger(__name__)


ProviderType = Literal[
    "anthropic",
    "groq",
    "mistral",
    "openai",
    "openrouter",
    "github",
    "copilot",
    "cerebras",
    "gemini",
    "cohere",
    "deepseek",
    "requesty",
    "xai",
    "comet",
    "novita",
    "vercel-gateway",
    "chutes",
    "cortecs",
    "azure",
    "fireworks-ai",
    "ollama",
    "moonshotai",
    "zai",
]


_PROVIDER_MAP: dict[ProviderType, Callable[[], ModelProvider]] = {
    "anthropic": AnthropicProvider,
    "groq": GroqProvider,
    "mistral": MistralProvider,
    "openai": OpenAIProvider,
    "openrouter": OpenRouterProvider,
    "github": GitHubProvider,
    "copilot": CopilotProvider,
    "cerebras": CerebrasProvider,
    "gemini": GeminiProvider,
    "cohere": CohereProvider,
    "deepseek": DeepSeekProvider,
    "requesty": RequestyProvider,
    "xai": XAIProvider,
    "comet": CometProvider,
    "novita": NovitaProvider,
    "vercel-gateway": VercelGatewayProvider,
    "fireworks-ai": FireworksProvider,
    "azure": AzureProvider,
    "chutes": ChutesProvider,
    "cortecs": CortecsProvider,
    "ollama": OllamaProvider,
    "zai": ZAIProvider,
    "moonshotai": MoonshotProvider,
}


def get_all_models_sync(
    *,
    providers: Sequence[ProviderType] | None = None,
    max_workers: int | None = None,
    include_deprecated: bool = False,
) -> list[ModelInfo]:
    """Fetch models from selected providers in parallel using threads.

    Args:
        providers: Sequence of provider names to use. Defaults to available providers.
        max_workers: Maximum number of worker threads.
                     Defaults to min(32, os.cpu_count() * 5)
        include_deprecated: Whether to include deprecated models. Defaults to False.

    Returns:
        list[ModelInfo]: Combined list of models from all providers.
    """
    import concurrent.futures

    if providers is not None:
        selected_providers = providers
    else:
        # Only use available providers when no specific providers are requested
        selected_providers = []
        for provider_name, provider_class in _PROVIDER_MAP.items():
            try:
                provider = provider_class()
                if provider.is_available():
                    selected_providers.append(provider_name)
            except Exception:  # noqa: BLE001
                # Provider initialization failed, skip it
                continue
    all_models: list[ModelInfo] = []

    def fetch_provider_models(provider_name: ProviderType) -> list[ModelInfo] | None:
        """Fetch models from a single provider."""
        import anyenv

        try:
            provider = _PROVIDER_MAP[provider_name]()
            models = anyenv.run_sync(provider.get_models())
            if not include_deprecated:
                models = [model for model in models if not model.is_deprecated]
            models = [model for model in models if not model.is_embedding]
        except Exception as e:  # noqa: BLE001
            logger.warning("Failed to fetch models from %s: %s", provider_name, str(e))
            return None
        else:
            return models

    with concurrent.futures.ThreadPoolExecutor(max_workers=max_workers) as executor:
        # Submit all tasks and collect results as they complete
        fut = {executor.submit(fetch_provider_models, p): p for p in selected_providers}
        for future in concurrent.futures.as_completed(fut):
            provider_models = future.result()
            if provider_models:
                all_models.extend(provider_models)

    successful_providers = {model.provider for model in all_models}
    logger.info(
        "Fetched %d models from %d providers: %s",
        len(all_models),
        len(successful_providers),
        ", ".join(sorted(successful_providers)),
    )
    return all_models


async def get_all_models(
    *,
    providers: Sequence[ProviderType] | None = None,
    include_deprecated: bool = False,
) -> list[ModelInfo]:
    """Fetch models from selected providers in parallel.

    Args:
        providers: Sequence of provider names to use. Defaults to available providers.
        include_deprecated: Whether to include deprecated models. Defaults to False.

    Returns:
        list[ModelInfo]: Combined list of models from all providers.
    """
    if providers is not None:
        selected_providers = providers
    else:
        # Only use available providers when no specific providers are requested
        selected_providers = []
        for provider_name, provider_class in _PROVIDER_MAP.items():
            try:
                provider = provider_class()
                if provider.is_available():
                    selected_providers.append(provider_name)
            except Exception:  # noqa: BLE001
                # Provider initialization failed, skip it
                continue
    all_models: list[ModelInfo] = []

    async def fetch_provider_models(
        provider_name: ProviderType,
    ) -> list[ModelInfo] | None:
        """Fetch models from a single provider."""
        try:
            provider = _PROVIDER_MAP[provider_name]()
            models = await provider.get_models()
            if not include_deprecated:
                models = [model for model in models if not model.is_deprecated]
            models = [model for model in models if not model.is_embedding]
        except Exception as e:  # noqa: BLE001
            logger.warning("Failed to fetch models from %s: %s", provider_name, str(e))
            return None
        else:
            return models

    # Fetch models from all providers in parallel
    results = await asyncio.gather(
        *(fetch_provider_models(provider) for provider in selected_providers),
        return_exceptions=False,
    )

    # Combine results, filtering out None values from failed providers
    for provider_models in results:
        if provider_models:
            all_models.extend(provider_models)

    successful_providers = {model.provider for model in all_models}
    logger.info(
        "Fetched %d models from %d providers: %s",
        len(all_models),
        len(successful_providers),
        ", ".join(sorted(successful_providers)),
    )
    return all_models


__all__ = [
    "AnthropicProvider",
    "CerebrasProvider",
    "CohereProvider",
    "CometProvider",
    "CopilotProvider",
    "DeepSeekProvider",
    "GeminiProvider",
    "GitHubProvider",
    "GroqProvider",
    "MistralProvider",
    "ModelInfo",
    "ModelPricing",
    "ModelProvider",
    "NovitaProvider",
    "OllamaProvider",
    "OpenAIProvider",
    "OpenRouterProvider",
    "ProviderType",
    "RequestyProvider",
    "VercelGatewayProvider",
    "XAIProvider",
    "get_all_models",
    "get_all_models_sync",
    "token_manager",
]



"""Generated model names from all providers.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total providers: 15
"""

from __future__ import annotations


from .anthropic import AnthropicModels
from .cerebras import CerebrasModels
from .comet import CometModels
from .copilot import CopilotModels
from .deepseek import DeepseekModels
from .gemini import GeminiModels
from .groq import GroqModels
from .mistral import MistralModels
from .moonshotai import MoonshotaiModels
from .openai import OpenaiModels
from .openrouter import OpenrouterModels
from .requesty import RequestyModels
from .vercel_gateway import Vercel_GatewayModels
from .xai import XaiModels
from .zai import ZaiModels

# Union of all provider model types
ModelName = (
    AnthropicModels
    | CerebrasModels
    | CometModels
    | CopilotModels
    | DeepseekModels
    | GeminiModels
    | GroqModels
    | MistralModels
    | MoonshotaiModels
    | OpenaiModels
    | OpenrouterModels
    | RequestyModels
    | Vercel_GatewayModels
    | XaiModels
    | ZaiModels
)

__all__ = [
    "AnthropicModels",
    "CerebrasModels",
    "CometModels",
    "CopilotModels",
    "DeepseekModels",
    "GeminiModels",
    "GroqModels",
    "MistralModels",
    "ModelName",
    "MoonshotaiModels",
    "OpenaiModels",
    "OpenrouterModels",
    "RequestyModels",
    "Vercel_GatewayModels",
    "XaiModels",
    "ZaiModels",
]



"""Token counting utilities with multiple fallback strategies."""

from __future__ import annotations

from functools import lru_cache
from importlib.util import find_spec
from typing import TYPE_CHECKING


if TYPE_CHECKING:
    from collections.abc import Sequence


@lru_cache
def has_library(name: str) -> bool:
    """Check if a Python package is available."""
    return bool(find_spec(name))


def count_tokens(text: str | Sequence[str], model: str | None = None) -> int:
    """Count total number of tokens in text(s) with fallback strategies.

    Uses following fallback order:
    1. tiktoken (if available)
    2. transformers (if available)
    3. rough approximation

    Args:
        text: Single text or sequence of texts to count tokens for
        model: Optional model name for tiktoken (ignored in fallbacks)

    Returns:
        Total token count for all provided text(s)
    """
    # Try tiktoken first
    if has_library("tiktoken"):
        import tiktoken  # pyright: ignore[reportMissingImports]

        try:
            encoding = tiktoken.encoding_for_model(model or "gpt-3.5-turbo")
        except KeyError:
            encoding = tiktoken.encoding_for_model("gpt-3.5-turbo")

        if isinstance(text, str):
            return len(encoding.encode(text))

        return sum(len(encoding.encode(t)) for t in text)

    # Try transformers as fallback
    if has_library("transformers"):
        from transformers import AutoTokenizer  # pyright: ignore[reportMissingImports]

        tokenizer = AutoTokenizer.from_pretrained("facebook/opt-125m")

        if isinstance(text, str):
            return len(tokenizer.encode(text))

        return sum(len(tokenizer.encode(t)) for t in text)

    # Last resort: rough approximation
    if isinstance(text, str):
        return len(text.split()) + len(text) // 4

    return sum(len(t.split()) + len(t) // 4 for t in text)



"""Generated model names for xai provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 8
"""

from __future__ import annotations

from typing import Literal


XaiModels = Literal[
    "xai:grok-2-1212",
    "xai:grok-2-vision-1212",
    "xai:grok-3",
    "xai:grok-3-mini",
    "xai:grok-4-0709",
    "xai:grok-4-fast-non-reasoning",
    "xai:grok-4-fast-reasoning",
    "xai:grok-code-fast-1",
]

__all__ = ["XaiModels"]



"""Novita AI provider."""

from __future__ import annotations

import os
from typing import Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo, ModelPricing


class NovitaProvider(ModelProvider):
    """Novita AI API provider."""

    def __init__(self, api_key: str | None = None):
        super().__init__()
        self.api_key = api_key or os.environ.get("NOVITA_API_KEY")
        if not self.api_key:
            msg = "Novita API key not found in parameters or NOVITA_API_KEY env var"
            raise RuntimeError(msg)

        self.base_url = "https://api.novita.ai/openai/v1"
        self.headers = {"Authorization": f"Bearer {self.api_key}"}
        self.params = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.api_key)

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse Novita API response into ModelInfo."""
        # Convert pricing from per million tokens to per token
        pricing = ModelPricing(
            prompt=float(data["input_token_price_per_m"]) / 1_000_000
            if data.get("input_token_price_per_m")
            else None,
            completion=float(data["output_token_price_per_m"]) / 1_000_000
            if data.get("output_token_price_per_m")
            else None,
        )

        # Build metadata for additional fields
        metadata = {}
        if "created" in data:
            metadata["created_timestamp"] = data["created"]
        if "object" in data:
            metadata["object_type"] = data["object"]

        return ModelInfo(
            id=str(data["id"]),
            name=str(data.get("title", data["id"])),
            provider="novita",
            description=str(data.get("description")) if data.get("description") else None,
            context_window=int(data["context_size"])
            if data.get("context_size")
            else None,
            pricing=pricing,
            metadata=metadata,
        )


if __name__ == "__main__":
    import asyncio

    async def main():
        provider = NovitaProvider()
        models = await provider.get_models()
        for model in models:
            print(model.format())

    asyncio.run(main())



"""Cohere provider."""

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


class CohereProvider(ModelProvider):
    """Cohere API provider."""

    def __init__(self, api_key: str | None = None):
        super().__init__()
        self.api_key = (
            api_key or os.environ.get("COHERE_API_KEY") or os.environ.get("CO_API_KEY")
        )
        if not self.api_key:
            msg = "Cohere API key not found in parameters or COHERE_API_KEY env var"
            raise RuntimeError(msg)

        self.base_url = "https://api.cohere.com/v1"
        self.headers = {"Authorization": f"Bearer {self.api_key}"}
        self.params = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.api_key)

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse Cohere API response into ModelInfo."""
        # Extract model name and ID
        name = data.get("name", "")

        # Determine supported modalities
        input_modalities: set[Modality] = {"text"}
        if data.get("supports_vision"):
            input_modalities.add("image")

        # Extract context length if available
        context_window = None
        if isinstance(data.get("context_length"), int | float):
            context_window = int(data["context_length"])

        # Extract endpoints
        endpoints = data.get("endpoints", [])
        endpoints_str = ", ".join(endpoints) if endpoints else ""

        # Build description
        description = []
        if endpoints_str:
            description.append(f"Supported endpoints: {endpoints_str}")
        if data.get("tokenizer_url"):
            description.append(f"Tokenizer URL: {data['tokenizer_url']}")
        if data.get("finetuned"):
            description.append("This is a fine-tuned model")

        description_str = "\n".join(description) if description else None
        is_embedding = "embed" in data.get("endpoints", []) or "embed_image" in data.get(
            "endpoints", []
        )
        return ModelInfo(
            id=name,
            name=name,
            provider="cohere",
            description=description_str,
            context_window=context_window,
            is_embedding=is_embedding,
            input_modalities=input_modalities,
        )

    async def get_models(self) -> list[ModelInfo]:
        """Override the base method to handle Cohere's unique API structure."""
        from anyenv import HttpError, get_json

        url = f"{self.base_url}/models"

        try:
            data = await get_json(
                url,
                headers=self.headers,
                params=self.params,
                cache=True,
                return_type=dict,
            )

            if "models" not in data:
                msg = "Invalid response format from Cohere API"
                raise RuntimeError(msg)

            return [self._parse_model(item) for item in data["models"]]

        except HttpError as e:
            msg = f"Failed to fetch models from Cohere: {e}"
            raise RuntimeError(msg) from e


if __name__ == "__main__":
    import asyncio

    async def main():
        provider = CohereProvider()
        models = await provider.get_models()
        for model in models:
            print(model.format())

    asyncio.run(main())



"""Generated model names for cerebras provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 4
"""

from __future__ import annotations

from typing import Literal


CerebrasModels = Literal[
    "cerebras:gpt-oss-120b",
    "cerebras:qwen-3-235b-a22b-instruct-2507",
    "cerebras:qwen-3-coder-480b",
    "cerebras:zai-glm-4.6",
]

__all__ = ["CerebrasModels"]



"""Mistral provider."""

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


class MistralProvider(ModelProvider):
    """Mistral AI API provider."""

    def __init__(self, api_key: str | None = None):
        super().__init__()
        self.api_key = api_key or os.environ.get("MISTRAL_API_KEY")
        if not self.api_key:
            msg = "Mistral API key not found in parameters or MISTRAL_API_KEY env var"
            raise RuntimeError(msg)

        self.base_url = "https://api.mistral.ai/v1"
        self.headers = {"Authorization": f"Bearer {self.api_key}"}
        self.params = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.api_key)

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse Mistral API response into ModelInfo."""
        # Extract capabilities
        capabilities = data.get("capabilities", {})

        # Determine if model is an embedding model
        is_embedding = False
        model_id = str(data["id"]).lower()
        model_name = str(data.get("name", data["id"])).lower()

        if (
            "embed" in model_id
            or "embed" in model_name
            or (
                not capabilities.get("completion_chat", True)
                and not capabilities.get("function_calling", True)
            )
        ):
            is_embedding = True

        # Determine input/output modalities
        input_modalities: set[Modality] = {"text"}
        if capabilities.get("vision"):
            input_modalities.add("image")

        return ModelInfo(
            id=str(data["id"]),
            name=str(data.get("name", data["id"])),
            provider="mistral",
            owned_by=str(data.get("owned_by")),
            description=str(data.get("description")),
            context_window=int(data.get("max_context_length", 0)),
            is_deprecated=bool(data.get("deprecation")),
            is_embedding=is_embedding,
            input_modalities=input_modalities,
        )


if __name__ == "__main__":
    import asyncio

    provider = MistralProvider()
    models = asyncio.run(provider.get_models())
    for model in models:
        print(model.format())



"""Modern Pydantic models for LLM pricing configuration using discriminated unions."""

from __future__ import annotations

import json
import pathlib
from typing import TYPE_CHECKING, Any

from schemez import Schema

from tokonomics import log
from tokonomics.data_models import (
    AudioSpeechModel,
    AudioTranscriptionModel,
    ChatCompletionModel,
    EmbeddingModel,
    ImageGenerationModel,
    ModelConfig,
    ModerationModel,
    RerankModel,
    ResponsesModel,
    VideoGenerationModel,
)


logger = log.get_logger(__name__)

if TYPE_CHECKING:
    from collections.abc import Iterator, Mapping

    from tokonomics.data_models import ModelMode


def iter_model_name_candidates(name: str) -> Iterator[str]:
    """Iter through model name candidates in LiteLLM pricing data.

    Iterates through name candidates for the LLM pricing data
    by trying different formats (direct match, base name, provider format).

    Args:
        name: Model name (e.g. "openai:gpt-4", "gpt-4")

    Yields:
        str: Name candidates for the LLM pricing data
    """
    name = name.lower()
    yield name
    if ":" in name:  # For provider:name format, try both variants
        provider, model_name = name.split(":", 1)
        yield model_name
        yield f"{provider.lower()}/{model_name}"


class ModelRegistry(Schema):
    """Registry containing all model configurations with query capabilities."""

    models: dict[str, ModelConfig]
    """Model name -> configuration mapping."""

    def get_model(self, name: str) -> ModelConfig | None:
        """Get model configuration by name."""
        return self.models.get(name)

    def get_models_by_provider(self, provider: str) -> dict[str, ModelConfig]:
        """Get all models for a specific provider."""
        return {
            name: config
            for name, config in self.models.items()
            if config.litellm_provider == provider
        }

    def get_models_by_mode(self, mode: ModelMode) -> dict[str, ModelConfig]:
        """Get all models for a specific mode."""
        return {
            name: config for name, config in self.models.items() if config.mode == mode
        }

    def get_chat_models(self) -> Mapping[str, ChatCompletionModel]:
        """Get all chat/completion models."""
        return {
            name: config
            for name, config in self.models.items()
            if isinstance(config, ChatCompletionModel)
        }

    def get_providers(self) -> set[str]:
        """Get all unique providers."""
        return {config.litellm_provider for config in self.models.values()}

    @classmethod
    def from_dict(cls, data: dict[str, Any]) -> ModelRegistry:
        """Create registry from dictionary data with automatic model discrimination."""
        models = {}
        failed_models = []
        dropped_models = []

        for name, config_data in data.items():
            try:
                # Skip models without mode field or sample_spec
                if "mode" not in config_data or name == "sample_spec":
                    dropped_models.append(name)
                    continue

                mode = config_data["mode"]
                match mode:
                    case "chat" | "completion":
                        model_config: ModelConfig = ChatCompletionModel.model_validate(
                            config_data
                        )
                    case "embedding":
                        model_config = EmbeddingModel.model_validate(config_data)
                    case "audio_transcription":
                        model_config = AudioTranscriptionModel.model_validate(config_data)
                    case "audio_speech":
                        model_config = AudioSpeechModel.model_validate(config_data)
                    case "image_generation":
                        model_config = ImageGenerationModel.model_validate(config_data)
                    case "video_generation":
                        model_config = VideoGenerationModel.model_validate(config_data)
                    case "rerank":
                        model_config = RerankModel.model_validate(config_data)
                    case "responses":
                        model_config = ResponsesModel.model_validate(config_data)
                    case "moderation":
                        model_config = ModerationModel.model_validate(config_data)
                    case _:
                        # Default to chat for unknown modes
                        model_config = ChatCompletionModel.model_validate(config_data)

                models[name] = model_config

            except Exception as e:  # noqa: BLE001
                failed_models.append((name, str(e)))
                # Only print first few failures to avoid spam
                if len(failed_models) <= 5:  # noqa: PLR2004
                    logger.debug("Failed to parse model %s: %s", name, e)

        if dropped_models:
            logger.debug("Dropped %s models without mode field", len(dropped_models))
        if failed_models:
            msg = "Failed to parse %s models out of %s"
            logger.debug(msg, len(failed_models), len(data))
        return cls(models=models)

    @classmethod
    def from_json_file(cls, file_path: str) -> ModelRegistry:
        """Load registry from JSON file."""
        with pathlib.Path(file_path).open() as f:
            data = json.load(f)
        return cls.from_dict(data)


def load_model_config(config_data: dict[str, Any]) -> ModelConfig:  # noqa: PLR0911
    """Load a single model configuration with manual discrimination."""
    mode = config_data.get("mode", "chat")

    match mode:
        case "chat" | "completion":
            return ChatCompletionModel.model_validate(config_data)
        case "embedding":
            return EmbeddingModel.model_validate(config_data)
        case "audio_transcription":
            return AudioTranscriptionModel.model_validate(config_data)
        case "audio_speech":
            return AudioSpeechModel.model_validate(config_data)
        case "image_generation":
            return ImageGenerationModel.model_validate(config_data)
        case "video_generation":
            return VideoGenerationModel.model_validate(config_data)
        case "rerank":
            return RerankModel.model_validate(config_data)
        case "responses":
            return ResponsesModel.model_validate(config_data)
        case "moderation":
            return ModerationModel.model_validate(config_data)
        case _:
            return ChatCompletionModel.model_validate(config_data)



"""Copilot model info Provider."""

from tokonomics.model_discovery.copilot_provider.provider import CopilotProvider
from tokonomics.model_discovery.copilot_provider.token_manager import CopilotTokenManager


__all__ = ["CopilotProvider", "CopilotTokenManager"]



"""Generated model names for openrouter provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 342
"""

from __future__ import annotations

from typing import Literal


OpenrouterModels = Literal[
    "openrouter:agentica-org/deepcoder-14b-preview",
    "openrouter:agentica-org/deepcoder-14b-preview:free",
    "openrouter:ai21/jamba-large-1.7",
    "openrouter:ai21/jamba-mini-1.7",
    "openrouter:aion-labs/aion-1.0",
    "openrouter:aion-labs/aion-1.0-mini",
    "openrouter:aion-labs/aion-rp-llama-3.1-8b",
    "openrouter:alfredpros/codellama-7b-instruct-solidity",
    "openrouter:alibaba/tongyi-deepresearch-30b-a3b",
    "openrouter:alibaba/tongyi-deepresearch-30b-a3b:free",
    "openrouter:allenai/olmo-2-0325-32b-instruct",
    "openrouter:alpindale/goliath-120b",
    "openrouter:amazon/nova-lite-v1",
    "openrouter:amazon/nova-micro-v1",
    "openrouter:amazon/nova-premier-v1",
    "openrouter:amazon/nova-pro-v1",
    "openrouter:anthracite-org/magnum-v4-72b",
    "openrouter:anthropic/claude-3-haiku",
    "openrouter:anthropic/claude-3-opus",
    "openrouter:anthropic/claude-3.5-haiku",
    "openrouter:anthropic/claude-3.5-haiku-20241022",
    "openrouter:anthropic/claude-3.5-sonnet",
    "openrouter:anthropic/claude-3.7-sonnet",
    "openrouter:anthropic/claude-3.7-sonnet:thinking",
    "openrouter:anthropic/claude-haiku-4.5",
    "openrouter:anthropic/claude-opus-4",
    "openrouter:anthropic/claude-opus-4.1",
    "openrouter:anthropic/claude-sonnet-4",
    "openrouter:anthropic/claude-sonnet-4.5",
    "openrouter:arcee-ai/afm-4.5b",
    "openrouter:arcee-ai/coder-large",
    "openrouter:arcee-ai/maestro-reasoning",
    "openrouter:arcee-ai/spotlight",
    "openrouter:arcee-ai/virtuoso-large",
    "openrouter:arliai/qwq-32b-arliai-rpr-v1",
    "openrouter:arliai/qwq-32b-arliai-rpr-v1:free",
    "openrouter:baidu/ernie-4.5-21b-a3b",
    "openrouter:baidu/ernie-4.5-21b-a3b-thinking",
    "openrouter:baidu/ernie-4.5-300b-a47b",
    "openrouter:baidu/ernie-4.5-vl-28b-a3b",
    "openrouter:baidu/ernie-4.5-vl-424b-a47b",
    "openrouter:bytedance/ui-tars-1.5-7b",
    "openrouter:cognitivecomputations/dolphin-mistral-24b-venice-edition:free",
    "openrouter:cohere/command-a",
    "openrouter:cohere/command-r-08-2024",
    "openrouter:cohere/command-r-plus-08-2024",
    "openrouter:cohere/command-r7b-12-2024",
    "openrouter:deepcogito/cogito-v2-preview-deepseek-671b",
    "openrouter:deepcogito/cogito-v2-preview-llama-109b-moe",
    "openrouter:deepcogito/cogito-v2-preview-llama-405b",
    "openrouter:deepcogito/cogito-v2-preview-llama-70b",
    "openrouter:deepseek/deepseek-chat",
    "openrouter:deepseek/deepseek-chat-v3-0324",
    "openrouter:deepseek/deepseek-chat-v3-0324:free",
    "openrouter:deepseek/deepseek-chat-v3.1",
    "openrouter:deepseek/deepseek-chat-v3.1:free",
    "openrouter:deepseek/deepseek-prover-v2",
    "openrouter:deepseek/deepseek-r1",
    "openrouter:deepseek/deepseek-r1-0528",
    "openrouter:deepseek/deepseek-r1-0528-qwen3-8b",
    "openrouter:deepseek/deepseek-r1-0528-qwen3-8b:free",
    "openrouter:deepseek/deepseek-r1-0528:free",
    "openrouter:deepseek/deepseek-r1-distill-llama-70b",
    "openrouter:deepseek/deepseek-r1-distill-llama-70b:free",
    "openrouter:deepseek/deepseek-r1-distill-qwen-14b",
    "openrouter:deepseek/deepseek-r1-distill-qwen-32b",
    "openrouter:deepseek/deepseek-r1:free",
    "openrouter:deepseek/deepseek-v3.1-terminus",
    "openrouter:deepseek/deepseek-v3.1-terminus:exacto",
    "openrouter:deepseek/deepseek-v3.2-exp",
    "openrouter:eleutherai/llemma_7b",
    "openrouter:google/gemini-2.0-flash-001",
    "openrouter:google/gemini-2.0-flash-exp:free",
    "openrouter:google/gemini-2.0-flash-lite-001",
    "openrouter:google/gemini-2.5-flash",
    "openrouter:google/gemini-2.5-flash-image",
    "openrouter:google/gemini-2.5-flash-image-preview",
    "openrouter:google/gemini-2.5-flash-lite",
    "openrouter:google/gemini-2.5-flash-lite-preview-06-17",
    "openrouter:google/gemini-2.5-flash-lite-preview-09-2025",
    "openrouter:google/gemini-2.5-flash-preview-09-2025",
    "openrouter:google/gemini-2.5-pro",
    "openrouter:google/gemini-2.5-pro-preview",
    "openrouter:google/gemini-2.5-pro-preview-05-06",
    "openrouter:google/gemma-2-27b-it",
    "openrouter:google/gemma-2-9b-it",
    "openrouter:google/gemma-3-12b-it",
    "openrouter:google/gemma-3-12b-it:free",
    "openrouter:google/gemma-3-27b-it",
    "openrouter:google/gemma-3-27b-it:free",
    "openrouter:google/gemma-3-4b-it",
    "openrouter:google/gemma-3-4b-it:free",
    "openrouter:google/gemma-3n-e2b-it:free",
    "openrouter:google/gemma-3n-e4b-it",
    "openrouter:google/gemma-3n-e4b-it:free",
    "openrouter:gryphe/mythomax-l2-13b",
    "openrouter:ibm-granite/granite-4.0-h-micro",
    "openrouter:inception/mercury",
    "openrouter:inception/mercury-coder",
    "openrouter:inclusionai/ling-1t",
    "openrouter:inclusionai/ring-1t",
    "openrouter:inflection/inflection-3-pi",
    "openrouter:inflection/inflection-3-productivity",
    "openrouter:kwaipilot/kat-coder-pro:free",
    "openrouter:liquid/lfm-2.2-6b",
    "openrouter:liquid/lfm2-8b-a1b",
    "openrouter:mancer/weaver",
    "openrouter:meituan/longcat-flash-chat",
    "openrouter:meituan/longcat-flash-chat:free",
    "openrouter:meta-llama/llama-3-70b-instruct",
    "openrouter:meta-llama/llama-3-8b-instruct",
    "openrouter:meta-llama/llama-3.1-405b",
    "openrouter:meta-llama/llama-3.1-405b-instruct",
    "openrouter:meta-llama/llama-3.1-70b-instruct",
    "openrouter:meta-llama/llama-3.1-8b-instruct",
    "openrouter:meta-llama/llama-3.2-11b-vision-instruct",
    "openrouter:meta-llama/llama-3.2-1b-instruct",
    "openrouter:meta-llama/llama-3.2-3b-instruct",
    "openrouter:meta-llama/llama-3.2-3b-instruct:free",
    "openrouter:meta-llama/llama-3.2-90b-vision-instruct",
    "openrouter:meta-llama/llama-3.3-70b-instruct",
    "openrouter:meta-llama/llama-3.3-70b-instruct:free",
    "openrouter:meta-llama/llama-4-maverick",
    "openrouter:meta-llama/llama-4-scout",
    "openrouter:meta-llama/llama-guard-2-8b",
    "openrouter:meta-llama/llama-guard-3-8b",
    "openrouter:meta-llama/llama-guard-4-12b",
    "openrouter:microsoft/mai-ds-r1",
    "openrouter:microsoft/mai-ds-r1:free",
    "openrouter:microsoft/phi-3-medium-128k-instruct",
    "openrouter:microsoft/phi-3-mini-128k-instruct",
    "openrouter:microsoft/phi-3.5-mini-128k-instruct",
    "openrouter:microsoft/phi-4",
    "openrouter:microsoft/phi-4-multimodal-instruct",
    "openrouter:microsoft/phi-4-reasoning-plus",
    "openrouter:microsoft/wizardlm-2-8x22b",
    "openrouter:minimax/minimax-01",
    "openrouter:minimax/minimax-m1",
    "openrouter:minimax/minimax-m2",
    "openrouter:mistralai/codestral-2501",
    "openrouter:mistralai/codestral-2508",
    "openrouter:mistralai/devstral-medium",
    "openrouter:mistralai/devstral-small",
    "openrouter:mistralai/devstral-small-2505",
    "openrouter:mistralai/magistral-medium-2506",
    "openrouter:mistralai/magistral-medium-2506:thinking",
    "openrouter:mistralai/magistral-small-2506",
    "openrouter:mistralai/ministral-3b",
    "openrouter:mistralai/ministral-8b",
    "openrouter:mistralai/mistral-7b-instruct",
    "openrouter:mistralai/mistral-7b-instruct-v0.1",
    "openrouter:mistralai/mistral-7b-instruct-v0.2",
    "openrouter:mistralai/mistral-7b-instruct-v0.3",
    "openrouter:mistralai/mistral-7b-instruct:free",
    "openrouter:mistralai/mistral-large",
    "openrouter:mistralai/mistral-large-2407",
    "openrouter:mistralai/mistral-large-2411",
    "openrouter:mistralai/mistral-medium-3",
    "openrouter:mistralai/mistral-medium-3.1",
    "openrouter:mistralai/mistral-nemo",
    "openrouter:mistralai/mistral-nemo:free",
    "openrouter:mistralai/mistral-saba",
    "openrouter:mistralai/mistral-small",
    "openrouter:mistralai/mistral-small-24b-instruct-2501",
    "openrouter:mistralai/mistral-small-24b-instruct-2501:free",
    "openrouter:mistralai/mistral-small-3.1-24b-instruct",
    "openrouter:mistralai/mistral-small-3.1-24b-instruct:free",
    "openrouter:mistralai/mistral-small-3.2-24b-instruct",
    "openrouter:mistralai/mistral-small-3.2-24b-instruct:free",
    "openrouter:mistralai/mistral-tiny",
    "openrouter:mistralai/mixtral-8x22b-instruct",
    "openrouter:mistralai/mixtral-8x7b-instruct",
    "openrouter:mistralai/pixtral-12b",
    "openrouter:mistralai/pixtral-large-2411",
    "openrouter:mistralai/voxtral-small-24b-2507",
    "openrouter:moonshotai/kimi-dev-72b",
    "openrouter:moonshotai/kimi-k2",
    "openrouter:moonshotai/kimi-k2-0905",
    "openrouter:moonshotai/kimi-k2-0905:exacto",
    "openrouter:moonshotai/kimi-k2-thinking",
    "openrouter:moonshotai/kimi-k2:free",
    "openrouter:moonshotai/kimi-linear-48b-a3b-instruct",
    "openrouter:morph/morph-v3-fast",
    "openrouter:morph/morph-v3-large",
    "openrouter:neversleep/llama-3.1-lumimaid-8b",
    "openrouter:neversleep/noromaid-20b",
    "openrouter:nousresearch/deephermes-3-mistral-24b-preview",
    "openrouter:nousresearch/hermes-2-pro-llama-3-8b",
    "openrouter:nousresearch/hermes-3-llama-3.1-405b",
    "openrouter:nousresearch/hermes-3-llama-3.1-405b:free",
    "openrouter:nousresearch/hermes-3-llama-3.1-70b",
    "openrouter:nousresearch/hermes-4-405b",
    "openrouter:nousresearch/hermes-4-70b",
    "openrouter:nvidia/llama-3.1-nemotron-70b-instruct",
    "openrouter:nvidia/llama-3.1-nemotron-ultra-253b-v1",
    "openrouter:nvidia/llama-3.3-nemotron-super-49b-v1.5",
    "openrouter:nvidia/nemotron-nano-12b-v2-vl",
    "openrouter:nvidia/nemotron-nano-12b-v2-vl:free",
    "openrouter:nvidia/nemotron-nano-9b-v2",
    "openrouter:nvidia/nemotron-nano-9b-v2:free",
    "openrouter:openai/chatgpt-4o-latest",
    "openrouter:openai/codex-mini",
    "openrouter:openai/gpt-3.5-turbo",
    "openrouter:openai/gpt-3.5-turbo-0613",
    "openrouter:openai/gpt-3.5-turbo-16k",
    "openrouter:openai/gpt-3.5-turbo-instruct",
    "openrouter:openai/gpt-4",
    "openrouter:openai/gpt-4-0314",
    "openrouter:openai/gpt-4-1106-preview",
    "openrouter:openai/gpt-4-turbo",
    "openrouter:openai/gpt-4-turbo-preview",
    "openrouter:openai/gpt-4.1",
    "openrouter:openai/gpt-4.1-mini",
    "openrouter:openai/gpt-4.1-nano",
    "openrouter:openai/gpt-4o",
    "openrouter:openai/gpt-4o-2024-05-13",
    "openrouter:openai/gpt-4o-2024-08-06",
    "openrouter:openai/gpt-4o-2024-11-20",
    "openrouter:openai/gpt-4o-audio-preview",
    "openrouter:openai/gpt-4o-mini",
    "openrouter:openai/gpt-4o-mini-2024-07-18",
    "openrouter:openai/gpt-4o-mini-search-preview",
    "openrouter:openai/gpt-4o-search-preview",
    "openrouter:openai/gpt-4o:extended",
    "openrouter:openai/gpt-5",
    "openrouter:openai/gpt-5-chat",
    "openrouter:openai/gpt-5-codex",
    "openrouter:openai/gpt-5-image",
    "openrouter:openai/gpt-5-image-mini",
    "openrouter:openai/gpt-5-mini",
    "openrouter:openai/gpt-5-nano",
    "openrouter:openai/gpt-5-pro",
    "openrouter:openai/gpt-5.1",
    "openrouter:openai/gpt-5.1-chat",
    "openrouter:openai/gpt-5.1-codex",
    "openrouter:openai/gpt-5.1-codex-mini",
    "openrouter:openai/gpt-oss-120b",
    "openrouter:openai/gpt-oss-120b:exacto",
    "openrouter:openai/gpt-oss-20b",
    "openrouter:openai/gpt-oss-20b:free",
    "openrouter:openai/gpt-oss-safeguard-20b",
    "openrouter:openai/o1",
    "openrouter:openai/o1-pro",
    "openrouter:openai/o3",
    "openrouter:openai/o3-deep-research",
    "openrouter:openai/o3-mini",
    "openrouter:openai/o3-mini-high",
    "openrouter:openai/o3-pro",
    "openrouter:openai/o4-mini",
    "openrouter:openai/o4-mini-deep-research",
    "openrouter:openai/o4-mini-high",
    "openrouter:opengvlab/internvl3-78b",
    "openrouter:openrouter/auto",
    "openrouter:openrouter/sherlock-dash-alpha",
    "openrouter:openrouter/sherlock-think-alpha",
    "openrouter:perplexity/sonar",
    "openrouter:perplexity/sonar-deep-research",
    "openrouter:perplexity/sonar-pro",
    "openrouter:perplexity/sonar-pro-search",
    "openrouter:perplexity/sonar-reasoning",
    "openrouter:perplexity/sonar-reasoning-pro",
    "openrouter:qwen/qwen-2.5-72b-instruct",
    "openrouter:qwen/qwen-2.5-72b-instruct:free",
    "openrouter:qwen/qwen-2.5-7b-instruct",
    "openrouter:qwen/qwen-2.5-coder-32b-instruct",
    "openrouter:qwen/qwen-2.5-coder-32b-instruct:free",
    "openrouter:qwen/qwen-2.5-vl-7b-instruct",
    "openrouter:qwen/qwen-max",
    "openrouter:qwen/qwen-plus",
    "openrouter:qwen/qwen-plus-2025-07-28",
    "openrouter:qwen/qwen-plus-2025-07-28:thinking",
    "openrouter:qwen/qwen-turbo",
    "openrouter:qwen/qwen-vl-max",
    "openrouter:qwen/qwen-vl-plus",
    "openrouter:qwen/qwen2.5-coder-7b-instruct",
    "openrouter:qwen/qwen2.5-vl-32b-instruct",
    "openrouter:qwen/qwen2.5-vl-32b-instruct:free",
    "openrouter:qwen/qwen2.5-vl-72b-instruct",
    "openrouter:qwen/qwen3-14b",
    "openrouter:qwen/qwen3-14b:free",
    "openrouter:qwen/qwen3-235b-a22b",
    "openrouter:qwen/qwen3-235b-a22b-2507",
    "openrouter:qwen/qwen3-235b-a22b-thinking-2507",
    "openrouter:qwen/qwen3-235b-a22b:free",
    "openrouter:qwen/qwen3-30b-a3b",
    "openrouter:qwen/qwen3-30b-a3b-instruct-2507",
    "openrouter:qwen/qwen3-30b-a3b-thinking-2507",
    "openrouter:qwen/qwen3-30b-a3b:free",
    "openrouter:qwen/qwen3-32b",
    "openrouter:qwen/qwen3-4b:free",
    "openrouter:qwen/qwen3-8b",
    "openrouter:qwen/qwen3-coder",
    "openrouter:qwen/qwen3-coder-30b-a3b-instruct",
    "openrouter:qwen/qwen3-coder-flash",
    "openrouter:qwen/qwen3-coder-plus",
    "openrouter:qwen/qwen3-coder:exacto",
    "openrouter:qwen/qwen3-coder:free",
    "openrouter:qwen/qwen3-max",
    "openrouter:qwen/qwen3-next-80b-a3b-instruct",
    "openrouter:qwen/qwen3-next-80b-a3b-thinking",
    "openrouter:qwen/qwen3-vl-235b-a22b-instruct",
    "openrouter:qwen/qwen3-vl-235b-a22b-thinking",
    "openrouter:qwen/qwen3-vl-30b-a3b-instruct",
    "openrouter:qwen/qwen3-vl-30b-a3b-thinking",
    "openrouter:qwen/qwen3-vl-8b-instruct",
    "openrouter:qwen/qwen3-vl-8b-thinking",
    "openrouter:qwen/qwq-32b",
    "openrouter:raifle/sorcererlm-8x22b",
    "openrouter:relace/relace-apply-3",
    "openrouter:sao10k/l3-euryale-70b",
    "openrouter:sao10k/l3-lunaris-8b",
    "openrouter:sao10k/l3.1-70b-hanami-x1",
    "openrouter:sao10k/l3.1-euryale-70b",
    "openrouter:sao10k/l3.3-euryale-70b",
    "openrouter:stepfun-ai/step3",
    "openrouter:switchpoint/router",
    "openrouter:tencent/hunyuan-a13b-instruct",
    "openrouter:thedrummer/anubis-70b-v1.1",
    "openrouter:thedrummer/cydonia-24b-v4.1",
    "openrouter:thedrummer/rocinante-12b",
    "openrouter:thedrummer/skyfall-36b-v2",
    "openrouter:thedrummer/unslopnemo-12b",
    "openrouter:thudm/glm-4.1v-9b-thinking",
    "openrouter:tngtech/deepseek-r1t-chimera",
    "openrouter:tngtech/deepseek-r1t-chimera:free",
    "openrouter:tngtech/deepseek-r1t2-chimera",
    "openrouter:tngtech/deepseek-r1t2-chimera:free",
    "openrouter:undi95/remm-slerp-l2-13b",
    "openrouter:x-ai/grok-3",
    "openrouter:x-ai/grok-3-beta",
    "openrouter:x-ai/grok-3-mini",
    "openrouter:x-ai/grok-3-mini-beta",
    "openrouter:x-ai/grok-4",
    "openrouter:x-ai/grok-4-fast",
    "openrouter:x-ai/grok-code-fast-1",
    "openrouter:z-ai/glm-4-32b",
    "openrouter:z-ai/glm-4.5",
    "openrouter:z-ai/glm-4.5-air",
    "openrouter:z-ai/glm-4.5-air:free",
    "openrouter:z-ai/glm-4.5v",
    "openrouter:z-ai/glm-4.6",
    "openrouter:z-ai/glm-4.6:exacto",
]

__all__ = ["OpenrouterModels"]



"""Helpers for calculating token costs using LiteLLM pricing data."""

from __future__ import annotations

from typing import Any


def _is_numeric(value: str | int | float | None) -> bool:  # noqa: PYI041
    """Check if a value can be converted to a number."""
    if value is None:
        return False
    if isinstance(value, int | float):
        return True
    if not isinstance(value, str):
        return False
    try:
        float(value)
    except ValueError:
        return False
    else:
        return True


def _safe_numeric_convert(value: Any) -> float:
    """Safely convert a value to float, returning 0 if not possible."""
    if value is None:
        return 0.0
    if isinstance(value, (int, float)):
        return float(value)
    if isinstance(value, str):
        try:
            return float(value)
        except ValueError:
            return 0.0
    return 0.0



"""Requesty provider."""

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo, ModelPricing


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


class RequestyProvider(ModelProvider):
    """Requesty API provider."""

    def __init__(self, api_key: str | None = None):
        super().__init__()
        self.api_key = api_key or os.environ.get("REQUESTY_API_KEY")

        self.base_url = "https://router.requesty.ai/v1"
        self.headers = {}
        if self.api_key:
            self.headers["Authorization"] = f"Bearer {self.api_key}"
        self.params = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.api_key)

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse Requesty API response into ModelInfo."""
        # Determine input modalities based on capabilities
        input_modalities: set[Modality] = {"text"}
        if data.get("supports_vision"):
            input_modalities.add("image")

        # Create pricing information
        pricing = ModelPricing(
            prompt=float(data["input_price"]) if data.get("input_price") else None,
            completion=float(data["output_price"]) if data.get("output_price") else None,
            input_cache_read=float(data["cached_price"])
            if data.get("cached_price")
            else None,
            input_cache_write=float(data["caching_price"])
            if data.get("caching_price")
            else None,
        )

        # Build metadata for additional capabilities
        metadata = {}
        if "supports_caching" in data:
            metadata["supports_caching"] = data["supports_caching"]
        if "supports_computer_use" in data:
            metadata["supports_computer_use"] = data["supports_computer_use"]
        if "supports_reasoning" in data:
            metadata["supports_reasoning"] = data["supports_reasoning"]
        if "created" in data:
            metadata["created_timestamp"] = data["created"]

        return ModelInfo(
            id=str(data["id"]),
            name=str(data["id"]),  # Requesty uses id as the model name
            provider="requesty",
            description=str(data.get("description")) if data.get("description") else None,
            pricing=pricing,
            owned_by=str(data.get("owned_by")) if data.get("owned_by") else None,
            context_window=int(data["context_window"])
            if data.get("context_window")
            else None,
            max_output_tokens=int(data["max_output_tokens"])
            if data.get("max_output_tokens")
            else None,
            input_modalities=input_modalities,
            metadata=metadata,
        )


if __name__ == "__main__":
    import asyncio

    provider = RequestyProvider()
    models = asyncio.run(provider.get_models())
    for model in models:
        print(model.format())



"""Generated model names for gemini provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 55
"""

from __future__ import annotations

from typing import Literal


GeminiModels = Literal[
    "gemini:aqa",
    "gemini:gemini-2.0-flash",
    "gemini:gemini-2.0-flash-001",
    "gemini:gemini-2.0-flash-exp",
    "gemini:gemini-2.0-flash-lite",
    "gemini:gemini-2.0-flash-lite-001",
    "gemini:gemini-2.0-flash-lite-preview",
    "gemini:gemini-2.0-flash-lite-preview-02-05",
    "gemini:gemini-2.0-flash-live-001",
    "gemini:gemini-2.0-flash-thinking-exp",
    "gemini:gemini-2.0-flash-thinking-exp-01-21",
    "gemini:gemini-2.0-flash-thinking-exp-1219",
    "gemini:gemini-2.0-pro-exp",
    "gemini:gemini-2.0-pro-exp-02-05",
    "gemini:gemini-2.5-computer-use-preview-10-2025",
    "gemini:gemini-2.5-flash",
    "gemini:gemini-2.5-flash-image",
    "gemini:gemini-2.5-flash-image-preview",
    "gemini:gemini-2.5-flash-lite",
    "gemini:gemini-2.5-flash-lite-preview-06-17",
    "gemini:gemini-2.5-flash-lite-preview-09-2025",
    "gemini:gemini-2.5-flash-live-preview",
    "gemini:gemini-2.5-flash-native-audio-latest",
    "gemini:gemini-2.5-flash-native-audio-preview-09-2025",
    "gemini:gemini-2.5-flash-preview-05-20",
    "gemini:gemini-2.5-flash-preview-09-2025",
    "gemini:gemini-2.5-flash-preview-tts",
    "gemini:gemini-2.5-pro",
    "gemini:gemini-2.5-pro-preview-03-25",
    "gemini:gemini-2.5-pro-preview-05-06",
    "gemini:gemini-2.5-pro-preview-06-05",
    "gemini:gemini-2.5-pro-preview-tts",
    "gemini:gemini-exp-1206",
    "gemini:gemini-flash-latest",
    "gemini:gemini-flash-lite-latest",
    "gemini:gemini-live-2.5-flash-preview",
    "gemini:gemini-pro-latest",
    "gemini:gemini-robotics-er-1.5-preview",
    "gemini:gemma-3-12b-it",
    "gemini:gemma-3-1b-it",
    "gemini:gemma-3-27b-it",
    "gemini:gemma-3-4b-it",
    "gemini:gemma-3n-e2b-it",
    "gemini:gemma-3n-e4b-it",
    "gemini:imagen-4.0-fast-generate-001",
    "gemini:imagen-4.0-generate-001",
    "gemini:imagen-4.0-generate-preview-06-06",
    "gemini:imagen-4.0-ultra-generate-001",
    "gemini:imagen-4.0-ultra-generate-preview-06-06",
    "gemini:learnlm-2.0-flash-experimental",
    "gemini:veo-2.0-generate-001",
    "gemini:veo-3.0-fast-generate-001",
    "gemini:veo-3.0-generate-001",
    "gemini:veo-3.1-fast-generate-preview",
    "gemini:veo-3.1-generate-preview",
]

__all__ = ["GeminiModels"]



"""Gemini provider."""

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any

from tokonomics.model_discovery.base import ModelProvider
from tokonomics.model_discovery.model_info import ModelInfo


if TYPE_CHECKING:
    from tokonomics.model_discovery.model_info import Modality


class GeminiProvider(ModelProvider):
    """Google Gemini API provider."""

    def __init__(self, api_key: str | None = None):
        super().__init__()
        self.api_key = api_key or os.environ.get("GEMINI_API_KEY")
        if not self.api_key:
            msg = "Gemini API key not found in parameters or GEMINI_API_KEY env var"
            raise RuntimeError(msg)

        self.base_url = "https://generativelanguage.googleapis.com/v1beta"
        self.params = {"key": self.api_key}
        self.headers = {}

    def is_available(self) -> bool:
        """Check whether the provider is available for use."""
        return bool(self.api_key)

    def _parse_model(self, data: dict[str, Any]) -> ModelInfo:
        """Parse Gemini API response into ModelInfo."""
        model_id = str(data.get("name", ""))
        if model_id.startswith("models/"):
            model_id = model_id[7:]  # Remove 'models/' prefix

        display_name = str(data.get("displayName", model_id))

        # Check for embedding model indicators
        is_embedding = False
        model_id_lower = model_id.lower()
        display_name_lower = display_name.lower()
        description = data.get("description", "").lower()

        # Check name/ID for embedding indicators
        if "embed" in model_id_lower or "embed" in display_name_lower:
            is_embedding = True

        # Check generation methods for embedding indicators
        generation_methods = data.get("supportedGenerationMethods", [])
        if any(method in ("embedContent", "embedText") for method in generation_methods):
            is_embedding = True

        # Check output token limit (embedding models typically output only 1 token)
        if data.get("outputTokenLimit") == 1:
            is_embedding = True

        # Check description for embedding indicators
        if "distributed representation" in description:
            is_embedding = True

        # Determine input/output modalities
        input_modalities: set[Modality] = {"text"}
        output_modalities: set[Modality] = {"text"}

        if "vision" in display_name_lower or "multimodal" in description:
            input_modalities.add("image")

        # Check for image generation capabilities
        if "image generation" in display_name_lower or "imagen" in model_id_lower:
            output_modalities.add("image")

        if "predict" in generation_methods and "imagen" in model_id_lower:
            output_modalities.add("image")

        return ModelInfo(
            id=model_id,
            name=display_name,
            provider="gemini",
            description=data.get("description"),
            owned_by="Google",
            context_window=data.get("inputTokenLimit"),
            max_output_tokens=data.get("outputTokenLimit"),
            input_modalities=input_modalities,
            output_modalities=output_modalities,
            is_embedding=is_embedding,
        )

    async def get_models(self) -> list[ModelInfo]:
        """Override the base method to handle Gemini's unique API structure."""
        from anyenv import HttpError, get_json

        url = f"{self.base_url}/models"

        try:
            data = await get_json(
                url,
                headers=self.headers,
                params=self.params,
                cache=True,
                return_type=dict,
            )

            if "models" not in data:
                msg = "Invalid response format from Gemini API"
                raise RuntimeError(msg)

            # Get all pages if nextPageToken is present
            models = data["models"]
            next_page_token = data.get("nextPageToken")

            while next_page_token:
                page_params = self.params.copy()
                page_params["pageToken"] = next_page_token

                next_data = await get_json(
                    url,
                    headers=self.headers,
                    params=page_params,
                    cache=True,
                    return_type=dict,
                )

                if "models" in next_data:
                    models.extend(next_data["models"])

                next_page_token = next_data.get("nextPageToken")

            return [self._parse_model(item) for item in models]

        except HttpError as e:
            msg = f"Failed to fetch models from Gemini: {e}"
            raise RuntimeError(msg) from e


if __name__ == "__main__":
    import asyncio

    async def main():
        provider = GeminiProvider()
        models = await provider.get_models()
        for model in models:
            print(model.format())

    asyncio.run(main())



"""Generated model names for copilot provider.

Auto-generated by scripts/generate_model_names.py - do not edit manually.
Total models: 5
"""

from __future__ import annotations

from typing import Literal


CopilotModels = Literal[
    "copilot:claude-haiku-4.5",
    "copilot:gpt-4.1",
    "copilot:gpt-4o",
    "copilot:gpt-5-mini",
    "copilot:oswe-vscode-prime",
]

__all__ = ["CopilotModels"]
